/*
 * Copyright (c) 2005 Broadcom Corporation
 *
 *    This driver was based on the sharpl NAND driver written by
 *    Richard Purdie.  Their copyright is below.
 *    As such, this driver falls under the GPL license also below.
 *
 *  Copyright (C) 2004 Richard Purdie
 *
 *  Based on Sharp's NAND driver sharp_sl.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */


#include <common.h>
#include <asm/errno.h>

#if defined(CONFIG_CMD_NAND) && defined(CONFIG_MTD_NAND_BCM_UMI)
#if !defined(CFG_NAND_LEGACY)


/*
 * Linux updated MTD structs, Linux driver updated but u-boot still
 * uses old mtd headers. Keep the updated sources for when they update.
 */
#define USE_OLD_MTD

#include <nand.h>

#include <linux/types.h>
// #include <linux/kernel.h>
#include <linux/string.h>
// #include <linux/ioport.h>
// #include <linux/device.h>
// #include <linux/delay.h>
//#include <linux/err.h>

//#include <linux/mtd/mtd.h>
//#include <linux/mtd/nand.h>
#include <linux/mtd/nand_ecc.h>
//#include <linux/mtd/partitions.h>
#include "linux/mtd/nand_ecc512.h"

#include <asm/io.h>
#include <asm/mach-types.h>

/*
#define REG_UMI_MMD_ICR_PHYS		__REG16(HW_IO_VIRT_TO_PHYS( REG_UMI_MMD_ICR ))
#define REG_UMI_NAND_TCR_PHYS		__REG32(HW_IO_VIRT_TO_PHYS( REG_UMI_NAND_TCR ))
#define REG_UMI_NAND_RCSR_PHYS		__REG8(	HW_IO_VIRT_TO_PHYS( REG_UMI_NAND_RCSR ))
#define REG_UMI_NAND_ECC_CSR_PHYS	__REG32(HW_IO_VIRT_TO_PHYS( REG_UMI_NAND_ECC_CSR ))
#define REG_UMI_NAND_ECC_DATA_PHYS	__REG32(HW_IO_VIRT_TO_PHYS( REG_UMI_NAND_ECC_DATA ))
#define REG_SYS_SUCR_PHYS			__REG32(HW_IO_VIRT_TO_PHYS( REG_SYS_SUCR ))
*/
/* Override Kernerl Macros: Virt -> Phys*/
/*
#undef NAND_BUS_16BIT()
#undef NAND_BUS_8BIT()
#undef NAND_WAIT_DONE()
#define NAND_BUS_16BIT()			( REG_SYS_SUCR_PHYS & REG_SYS_SUCR_NAND_BUS_WIDTH_16)
#define NAND_BUS_8BIT()				( !NAND_BUS_16BIT() )
#define NAND_WAIT_DONE()			{ while ((REG_UMI_NAND_RCSR_PHYS & REG_UMI_NAND_RCSR_RDY) == 0); }
*/


static char gBanner[] = "BCM UMI MTD NAND Driver: 1.00\n";

//  Controls whether we are using hardware ecc or software.
//

#ifdef CONFIG_MTD_NAND_BCM_UMI_HWECC
static int hardware_ecc = 1;
#else
static int hardware_ecc = 0;
#endif

//  Register offsets
//
#define REG_NAND_CMD_OFFSET     ( 0 )
#define REG_NAND_ADDR_OFFSET    ( 4 )
#define REG_NAND_DATA16_OFFSET  ( 8 )
#ifndef __ARMEB__
#define REG_NAND_DATA8_OFFSET   ( 8 )
#else
#define REG_NAND_DATA8_OFFSET   ( 8 + 1 )
#endif

#ifdef CONFIG_BCM116X_DMA
#include <asm/arch/dma.h>
#include <linux/dma-mapping.h>

#define NAND_DMA_CHAN   3

// Global config is common for both directions.
#define DMA_CFG                                 \
    ( REG_DMA_CHAN_CFG_TC_INT_ENABLE            \
    | REG_DMA_CHAN_CFG_ERROR_INT_ENABLE         \
    | REG_DMA_CHAN_CFG_FLOW_CTL_MEM_TO_MEM_DMA  \
    | REG_DMA_CHAN_CFG_ENABLE)

// Common transfer widths in bits (typically 8, 16, or 32)
#define DMA_WIDTH(dstwidth, srcwidth)                        \
    ( REG_DMA_CHAN_CTL_DEST_WIDTH_##dstwidth       \
    | REG_DMA_CHAN_CTL_SRC_WIDTH_##srcwidth )

// Common burst sizes - typically 4
#define DMA_BURST(width)                        \
    ( REG_DMA_CHAN_CTL_DEST_BURST_SIZE_##width  \
    | REG_DMA_CHAN_CTL_SRC_BURST_SIZE_##width )

// DMA settings for copying from NAND to SDRAM
#define DMA_CTRL_NAND_TO_SDRAM(dstwidth, srcwidth, bytes)    \
    ( REG_DMA_CHAN_CTL_TC_INT_ENABLE            \
    | REG_DMA_CHAN_CTL_DEST_INCR                \
    | DMA_BURST(4)                              \
    | DMA_WIDTH(dstwidth, srcwidth)             \
    | ( bytes * 8 / srcwidth ))

// DMA settings for copying from SDRAM to NAND
#define DMA_CTRL_SDRAM_TO_NAND(dstwidth, srcwidth, bytes)    \
    ( REG_DMA_CHAN_CTL_TC_INT_ENABLE            \
    | REG_DMA_CHAN_CTL_SRC_INCR                 \
    | DMA_BURST(4)                              \
    | DMA_WIDTH(dstwidth, srcwidth)             \
    | ( bytes * 8 / srcwidth ))
#endif

/****************************************************************************
*
*  nand_hw_eccoob
*
*   New oob placement block for use with hardware ecc generation.
*
***************************************************************************/

#ifdef USE_OLD_MTD
static struct nand_oobinfo nand_hw_eccoob = {
	.useecc		= MTD_NANDECC_AUTOPLACE,
#else
static struct nand_ecclayout nand_hw_eccoob = {
#endif
	.eccbytes	= 3,
	.eccpos		= {6, 7, 8 },
    // Reserve 0/1 and 10/11 as BI indicators for 16-bit flash
    // Reserve 5 for 8-bit BI
    // 6/7/8 are for ecc so this is all that's left
	.oobfree	= { {2, 3}, {9, 1}, {12, 4} }
};

/*
 * MTD structure for BCM UMI
 */
// static struct mtd_info *board_mtd = NULL;
 static void __iomem *bcm_io_base;

// Preallocate a buffer to avoid having to do this every dma operation.
// This is the size of the preallocated coherent DMA buffer.
#ifdef CONFIG_BCM116X_DMA
//#define DMA_MAX_BUFLEN     512
#define DMA_MAX_BUFLEN     0
static void *virtPtr;
static dma_addr_t physPtr;
#else
#define DMA_MAX_BUFLEN     0        // disable linux nand dma
#endif

/****************************************************************************
*
*  nand_dev_raedy
*
*   Routine to check if nand is ready.
*
***************************************************************************/
static int nand_dev_ready(struct mtd_info* mtd)
{
    return (REG_UMI_NAND_RCSR & REG_UMI_NAND_RCSR_RDY);
}

/****************************************************************************
*
*  bcm1161_nand_inithw
*
*   This routine does the necessary hardware (board-specific)
*   initializations.  This includes setting up the timings, etc.
*
***************************************************************************/

int bcm1161_nand_inithw( void )
{
    // Configure nand timing parameters
    // waiten = 0
    // lowfreq = 0
    // memtype async r/w, no page mode = 000
    // pgsz = 000
    // tprc_twlc = 00000     Page read access cycle / Burst write latency (n+2 / n+1)
    // tbta = 001            Bus turnaround cycle (n)
    // twp = 00100           Write pulse width cycle (n+1)
    // twr = 00              Write recovery cycle (n+1)
    // tas = 00              Write address setup cycle (n+1)
    // toe = 00              Output enable delay cycle (n)
    // trc_tlc = 00011       Read access cycle / Burst read latency (n+2 / n+1)
    REG_UMI_FLASH0_TCR = 0x00012003;

    // Configure nand timing parameters
    // tbta = 001            Bus turnaround cycle (n)
    // twp = 00100           Write pulse width cycle (n+1)
    // twr = 00              Write recovery cycle (n+1)
    // tas = 00              Write address setup cycle (n+1)
    // toe = 00              Output enable delay cycle (n)
    // trc_tlc = 00011       Read access cycle / Burst read latency (n+2 / n+1)
    REG_UMI_NAND_TCR &= ~0x7ffff;
    REG_UMI_NAND_TCR |=  0x72003;

    REG_UMI_NAND_TCR  |= REG_UMI_NAND_TCR_CS_SWCTRL;        // enable software control of CS
    REG_UMI_NAND_RCSR |= REG_UMI_NAND_RCSR_CS_ASSERTED;     // keep NAND chip select asserted

    if (NAND_BUS_16BIT())
    {
    	REG_UMI_NAND_TCR |= REG_UMI_NAND_TCR_WORD16;
    }
    else
    {
    	REG_UMI_NAND_TCR &= ~REG_UMI_NAND_TCR_WORD16;
    }
    REG_UMI_MMD_ICR |= REG_UMI_MMD_ICR_FLASH_WP;        //enable writes to flash

    writel( NAND_CMD_RESET, CFG_NAND_BASE + REG_NAND_CMD_OFFSET );
    NAND_WAIT_DONE();

	return 0;
}

/****************************************************************************
*
*  bcm1161_nand_hwcontrol
*
*   Used to turn latch the proper register for access.
*
***************************************************************************/

#ifdef USE_OLD_MTD
static void bcm1161_nand_hwcontrol(struct mtd_info *mtd, int cmd)
{
	struct nand_chip *chip = mtd->priv;

	switch (cmd) {
        case NAND_CTL_SETNCE:
            //  We are not supporting chip select right now.
            break;

        case NAND_CTL_CLRNCE:
            //  We are not supporting chip select right now.
            break;

        case NAND_CTL_SETCLE:
            chip->IO_ADDR_W    = (void * __iomem)CFG_NAND_BASE + REG_NAND_CMD_OFFSET;
            break;

        case NAND_CTL_SETALE:
            chip->IO_ADDR_W    = (void * __iomem)CFG_NAND_BASE + REG_NAND_ADDR_OFFSET;
            break;

        default:
            chip->IO_ADDR_W    = (void * __iomem)CFG_NAND_BASE +
                ((NAND_BUS_16BIT()) ? REG_NAND_DATA16_OFFSET : REG_NAND_DATA8_OFFSET);
            break;
	}
}
#else
static void bcm1161_nand_hwcontrol(struct mtd_info *mtd, int cmd, unsigned int ctrl)
{
   /* TODO: send command to hardward */
	struct nand_chip *chip = mtd->priv;
   if (ctrl & NAND_CTRL_CHANGE) {
      if (ctrl & NAND_CLE) {
         chip->IO_ADDR_W    = (void * __iomem)CFG_NAND_BASE + REG_NAND_CMD_OFFSET;
         goto CMD;
      }
      if (ctrl & NAND_ALE) {
         chip->IO_ADDR_W    = (void * __iomem)CFG_NAND_BASE + REG_NAND_ADDR_OFFSET;
         goto CMD;
      }
      chip->IO_ADDR_W    = (void * __iomem)CFG_NAND_BASE +
         ((NAND_BUS_16BIT()) ? REG_NAND_DATA16_OFFSET : REG_NAND_DATA8_OFFSET);
   }

CMD:
   /* Send command to chip directly */
   if (cmd != NAND_CMD_NONE)
      writeb(cmd, chip->IO_ADDR_W);

}
#endif
/****************************************************************************
*
*  bcm1161_nand_get_hw_ecc
*
*   Used to get the hardware ECC.
*
***************************************************************************/

static int bcm1161_nand_get_hw_ecc(struct mtd_info *mtd,
				      const u_char *dat, u_char *ecc_code)
{
    unsigned long ecc = REG_UMI_NAND_ECC_DATA;
    ecc_code[2] = (ecc >> 16) & 0xff;
    ecc_code[1] = (ecc >>  8) & 0xff;
    ecc_code[0] = (ecc >>  0) & 0xff;

    return 0;
}

/****************************************************************************
*
*  bcm1161_nand_enable_hwecc
*
*   Used to turn on hardware ECC.
*
***************************************************************************/

static void bcm1161_nand_enable_hwecc(struct mtd_info *mtd, int mode)
{

	REG_UMI_NAND_ECC_CSR &= ~REG_UMI_NAND_ECC_CSR_ECC_ENABLE;   // disable and reset ECC
	REG_UMI_NAND_ECC_CSR |= REG_UMI_NAND_ECC_CSR_ECC_ENABLE;    // enable ECC
	REG_UMI_NAND_ECC_CSR &= ~REG_UMI_NAND_ECC_CSR_256BYTE;      // 512 byte page
}

/**
 * nand_write_buf - write buffer to chip
 * @mtd:	MTD device structure
 * @buf:	data buffer
 * @len:	number of bytes to write
 *
 * Default write function for 8bit buswith
 */
static void bcm1161_nand_write_buf(struct mtd_info *mtd, const u_char *buf, int len)
{
   if (len != DMA_MAX_BUFLEN)   // Bulk of operations are 512. This is dma pays off.
   {
      // Do it the old way if the buffer is small or too large. Probably quicker than starting and checking dma.
      int i;
      struct nand_chip *this = mtd->priv;

      for (i=0; i<len; i++)
      {
         writeb(buf[i], this->IO_ADDR_W);
      }
   }
#ifdef CONFIG_BCM116X_DMA
   else
   {
      int dmactrl;
      memcpy(virtPtr, buf, len);
      dmactrl = NAND_BUS_16BIT() ? DMA_CTRL_SDRAM_TO_NAND(16, 32, len): DMA_CTRL_SDRAM_TO_NAND(8, 32, len);
      dma_init_chan(NAND_DMA_CHAN);
      dma_setup_chan(NAND_DMA_CHAN,
                     (int)physPtr,
                     NAND_BUS_16BIT() ? REG_NAND_DATA16_PADDR : REG_NAND_DATA8_PADDR,
                     0,
                     dmactrl,
                     DMA_CFG);
      dma_poll_chan(NAND_DMA_CHAN);
   }
#endif
}

/**
 * nand_read_buf - read chip data into buffer
 * @mtd:	MTD device structure
 * @buf:	buffer to store date
 * @len:	number of bytes to read
 *
 * Default read function for 8bit buswith
 */

static void bcm1161_nand_read_buf(struct mtd_info *mtd, u_char *buf, int len)
{
   if (len != DMA_MAX_BUFLEN)
   {
      int i;
      struct nand_chip *this = mtd->priv;

      for (i=0; i<len; i++)
      {
         buf[i] = readb(this->IO_ADDR_R);
      }
   }
#ifdef CONFIG_BCM116X_DMA
   else
   {
      int dmactrl;
      dmactrl = NAND_BUS_16BIT() ? DMA_CTRL_NAND_TO_SDRAM(32, 16, len) : DMA_CTRL_NAND_TO_SDRAM(32, 8, len);
      dma_init_chan(NAND_DMA_CHAN);
      dma_setup_chan(NAND_DMA_CHAN,
                     NAND_BUS_16BIT() ? REG_NAND_DATA16_PADDR : REG_NAND_DATA8_PADDR,
                     (int)physPtr,
                     0,
                     dmactrl,
                     DMA_CFG);
      dma_poll_chan(NAND_DMA_CHAN);
      memcpy(buf, virtPtr, len);
   }
#endif
}

/**
 * bcm1161_nand_verify_buf - Verify chip data against buffer
 * @mtd:	MTD device structure
 * @buf:	buffer containing the data to compare
 * @len:	number of bytes to compare
 *
 * Default verify function for 8bit buswith
 */
static int bcm1161_nand_verify_buf(struct mtd_info *mtd, const u_char *buf, int len)
{
   if (len != DMA_MAX_BUFLEN)
   {
      int i;
      struct nand_chip *this = mtd->priv;
      for (i=0; i<len; i++)
      {
         if (buf[i] != readb(this->IO_ADDR_R))
         {
            return -EFAULT;
         }
      }
   }
#ifdef CONFIG_BCM116X_DMA
   else
   {
      int dmactrl;
      dmactrl = NAND_BUS_16BIT() ? DMA_CTRL_NAND_TO_SDRAM(32, 16, len) : DMA_CTRL_NAND_TO_SDRAM(32, 8, len);
      dma_init_chan(NAND_DMA_CHAN);
      dma_setup_chan(NAND_DMA_CHAN,
                     NAND_BUS_16BIT() ? REG_NAND_DATA16_PADDR : REG_NAND_DATA8_PADDR,
                     (int)physPtr,
                     0,
                     dmactrl,
                     DMA_CFG);
      dma_poll_chan(NAND_DMA_CHAN);
      if (memcmp(buf, virtPtr, len) != 0)
      {
         return -EFAULT;
      }
   }
#endif
   return 0;
}

/**
 * bcm1161_nand_write_buf16 - write buffer to chip
 * @mtd:	MTD device structure
 * @buf:	data buffer
 * @len:	number of bytes to write
 *
 * Default write function for 16bit buswith
 */
static void bcm1161_nand_write_buf16(struct mtd_info *mtd, const u_char *buf, int len)
{
   if (len != DMA_MAX_BUFLEN)   // Bulk of operations are 512. This is dma pays off.
   {
      int i;
      struct nand_chip *this = mtd->priv;
      u16 *p = (u16 *) buf;
      len >>= 1;

      for (i=0; i<len; i++)
         writew(p[i], this->IO_ADDR_W);
   }
#ifdef CONFIG_BCM116X_DMA
   else
   {
      int dmactrl;
      memcpy(virtPtr, buf, len);
      dmactrl = NAND_BUS_16BIT() ? DMA_CTRL_SDRAM_TO_NAND(16, 32, len): DMA_CTRL_SDRAM_TO_NAND(8, 32, len);
      dma_init_chan(NAND_DMA_CHAN);
      dma_setup_chan(NAND_DMA_CHAN,
                     (int)physPtr,
                     NAND_BUS_16BIT() ? REG_NAND_DATA16_PADDR : REG_NAND_DATA8_PADDR,
                     0,
                     dmactrl,
                     DMA_CFG);
      dma_poll_chan(NAND_DMA_CHAN);
   }
#endif
}

/**
 * bcm1161_nand_read_buf16 - read chip data into buffer
 * @mtd:	MTD device structure
 * @buf:	buffer to store date
 * @len:	number of bytes to read
 *
 * Default read function for 16bit buswith
 */
static void bcm1161_nand_read_buf16(struct mtd_info *mtd, u_char *buf, int len)
{
   if (len != DMA_MAX_BUFLEN)
   {
      int words;
      int i;
      struct nand_chip *this = mtd->priv;
      u16 *p = (u16 *) buf;

      words = (len + 1) / 2;   // e.g len may be odd. 1 byte must do 1 word read

      for (i=0; i<words; i++)
      {
         p[i] = readw(this->IO_ADDR_R);
      }
   }
#ifdef CONFIG_BCM116X_DMA
   else
   {
      int dmactrl;
      dmactrl = NAND_BUS_16BIT() ? DMA_CTRL_NAND_TO_SDRAM(32, 16, len) : DMA_CTRL_NAND_TO_SDRAM(32, 8, len);
      dma_init_chan(NAND_DMA_CHAN);
      dma_setup_chan(NAND_DMA_CHAN,
                     NAND_BUS_16BIT() ? REG_NAND_DATA16_PADDR : REG_NAND_DATA8_PADDR,
                     (int)physPtr,
                     0,
                     dmactrl,
                     DMA_CFG);
      dma_poll_chan(NAND_DMA_CHAN);
      memcpy(buf, virtPtr, len);
   }
#endif
}

/**
 * bcm1161_nand_verify_buf16 - Verify chip data against buffer
 * @mtd:	MTD device structure
 * @buf:	buffer containing the data to compare
 * @len:	number of bytes to compare
 *
 * Default verify function for 16bit buswith
 */
static int bcm1161_nand_verify_buf16(struct mtd_info *mtd, const u_char *buf, int len)
{
   if (len != DMA_MAX_BUFLEN)
   {
      int i;
      struct nand_chip *this = mtd->priv;
      u16 *p = (u16 *) buf;
      len >>= 1;

      for (i=0; i<len; i++)
         if (p[i] != readw(this->IO_ADDR_R))
            return -EFAULT;
   }
#ifdef CONFIG_BCM116X_DMA
   else
   {
      int dmactrl;
      dmactrl = NAND_BUS_16BIT() ? DMA_CTRL_NAND_TO_SDRAM(32, 16, len) : DMA_CTRL_NAND_TO_SDRAM(32, 8, len);
      dma_init_chan(NAND_DMA_CHAN);
      dma_setup_chan(NAND_DMA_CHAN,
                     NAND_BUS_16BIT() ? REG_NAND_DATA16_PADDR : REG_NAND_DATA8_PADDR,
                     (int)physPtr,
                     0,
                     dmactrl,
                     DMA_CFG);
      dma_poll_chan(NAND_DMA_CHAN);
      if (memcmp(buf, virtPtr, len) != 0)
      {
         return -EFAULT;
      }
   }
#endif
   return 0;
}

#ifdef CONFIG_MTD_PARTITIONS
const char *part_probes[] = { "cmdlinepart", NULL };
#endif


/****************************************************************************
*
*  bcm1161_nand_init
*
***************************************************************************/

int board_nand_init(struct nand_chip *nand)
{
   int busw;

   printk( gBanner );

	/* Allocate memory for MTD device structure and private data */
	//board_mtd = kmalloc(sizeof(struct mtd_info) + sizeof(struct nand_chip), GFP_KERNEL);
	/*if (!board_mtd) {
		printf ("Unable to allocate NAND MTD device structure.\n");
		return -ENOMEM;
	}*/

	/* map physical adress */

#ifdef CONFIG_ARCH_BCMRING
	bcm_io_base = ioremap(MM_ADDR_IO_NAND, 0x1000);
#else
	bcm_io_base = (void __iomem *)CFG_NAND_BASE; //ioremap(0x08000000, 0x1000);
#endif


	if(!CFG_NAND_BASE){
		printk("ioremap to access BCM1161 NAND chip failed\n");
		// kfree(board_mtd);
		return -EIO;
	}

	/* Get pointer to private data */
	// nand = (struct nand_chip *) (&board_mtd[1]);

	/* Initialize structures */
	// memset((char *) board_mtd, 0, sizeof(struct mtd_info));
	// memset((char *) this, 0, sizeof(struct nand_chip));

	/* Link the private data with the MTD structure */
	// board_mtd->priv = this;

   /* Initialize the NAND hardware.  */
   if (bcm1161_nand_inithw() < 0)
   {
      printk("BCM1161 NAND chip could not be initialized\n");
      // iounmap(bcm1161_io_base);
      // kfree(board_mtd);
      return -EIO;
   }

   /* Set address of NAND IO lines */
   nand->IO_ADDR_W    = (void * __iomem)CFG_NAND_BASE + ((NAND_BUS_16BIT()) ? REG_NAND_DATA16_OFFSET : REG_NAND_DATA8_OFFSET);
   nand->IO_ADDR_R    = (void * __iomem)CFG_NAND_BASE + ((NAND_BUS_16BIT()) ? REG_NAND_DATA16_OFFSET : REG_NAND_DATA8_OFFSET);

   /* Set command delay time, see datasheet for correct value */
   //nand->chip_delay   = 0;
   /* Assign the device ready function, if available */


   nand->dev_ready    = nand_dev_ready;
   nand->options	   = (NAND_BUS_16BIT()) ? NAND_BUSWIDTH_16 : 0;

   busw = nand->options & NAND_BUSWIDTH_16;

	nand->write_buf = busw ? bcm1161_nand_write_buf16 : bcm1161_nand_write_buf;
	nand->read_buf = busw ? bcm1161_nand_read_buf16 : bcm1161_nand_read_buf;
	nand->verify_buf = busw ? bcm1161_nand_verify_buf16 : bcm1161_nand_verify_buf;

#ifdef USE_OLD_MTD
	/* Reference hardware control function */

	nand->hwcontrol    = bcm1161_nand_hwcontrol;
	if (hardware_ecc) {
      nand->correct_data  = nand_correct_data512;
		nand->enable_hwecc  = bcm1161_nand_enable_hwecc;
		nand->calculate_ecc = bcm1161_nand_get_hw_ecc;
		nand->eccmode	    = NAND_ECC_HW3_512;
		nand->autooob       = &nand_hw_eccoob;
	} else {
		nand->eccmode	    = NAND_ECC_SOFT;
	}
#else
   nand->cmd_ctrl = bcm1161_nand_hwcontrol;
   if (hardware_ecc) {
      nand->ecc.mode = NAND_ECC_HW;
      nand->ecc.size = 512;
      nand->ecc.layout = &nand_hw_eccoob;
      nand->ecc.bytes = 3;
      nand->ecc.correct = nand_correct_data512;
      nand->ecc.calculate = bcm1161_nand_get_hw_ecc;
      nand->ecc.hwctl = bcm1161_nand_enable_hwecc;
   } else {
      nand->ecc.mode = NAND_ECC_SOFT;
   }

#endif

#ifdef CONFIG_BCM116X_DMA
   if (DMA_MAX_BUFLEN)
   {
      dma_request_chan(NAND_DMA_CHAN, "nand");
      virtPtr = dma_alloc_coherent( NULL, DMA_MAX_BUFLEN, &physPtr, GFP_KERNEL );
      if ( virtPtr == NULL )
      {
         printk("NAND - Failed to allocate memory for DMA buffer\n");
         return -ENOMEM;
      }
   }
#endif

   /* Scan to find existence of the device */
	//err = nand_scan( board_mtd, 1 );
	//if (err)
    //{
/*		iounmap(bcm1161_io_base);
		kfree(board_mtd);
		return err;
*/
		//printf("Cannot find device");
	//}

	/* Register the partitions */
/*
{
   int nr_partitions;
	struct mtd_partition* partition_info;

	board_mtd->name = "bcm1161-nand";
	nr_partitions = parse_mtd_partitions(board_mtd, part_probes, &partition_info, 0);

	if (nr_partitions <= 0)
    {
		printk("BCM1161 NAND: Too few partitions - %d\n", nr_partitions);
		iounmap(bcm1161_io_base);
		kfree(board_mtd);
		return -EIO;
	}

	add_mtd_partitions( board_mtd, partition_info, nr_partitions );
}
*/

	/* Return happy */
	return 0;
}

/****************************************************************************/

/* module_init(bcm1161_nand_init); */

#ifdef MODULE
/****************************************************************************
*
*  bcm1161_nand_exit
*
***************************************************************************/
static void __exit bcm1161_nand_exit(void)
{
#ifdef CONFIG_BCM116X_DMA
   dma_free_chan(NAND_DMA_CHAN);
   if (DMA_MAX_BUFLEN)
   {
      dma_free_coherent( NULL, DMA_MAX_BUFLEN, virtPtr, physPtr );
   }
#endif

   /* Release resources, unregister device */
	nand_release(board_mtd);

	/* unmap physical adress */
	iounmap(bcm1161_io_base);

	/* Free the MTD device structure */
	kfree(board_mtd);
}
module_exit(bcm1161_nand_exit);
#endif
#endif
#endif
/*
MODULE_LICENSE("GPL");
MODULE_AUTHOR("Broadcom");
MODULE_DESCRIPTION("BCM UMI MTD NAND driver");
*/
