/*
 * (C) Copyright 2009 TomTom BV <http://www.tomtom.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * Implementation of the flip-flop. This can store 1 bit of information
 * across a (watchdog) reset.
 */

#include <common.h>
#include <regs.h>
#include <asm/io.h>
#include <flipflop.h>

#ifdef CONFIG_SW_FLIPFLOP

#define PA_FLAG_LOCATION	0x52000000

void flipflop_set(int v)
{
	volatile unsigned long *flg = (unsigned long *)PA_FLAG_LOCATION;
	if (v) {
		*flg = 0xdeadbeef;
	} else {
		*flg = 0x0;
	}
}

int flipflop_get(void)
{
	volatile unsigned long *flg = (unsigned long *)PA_FLAG_LOCATION;
	return (*flg == 0xdeadbeef);
}

#else /* CONFIG_SW_FLIPFLOP */

static int is_setup = 0;

static void flipflop_setup(void)
{
	u32 reg;
	if (is_setup)
		return;

	/* Set GPB(4) as output (clock) */
	reg = readl(GPBCON) & (~0xF0000);
	writel(reg | 0x10000, GPBCON);

	/* Set disable pullup/down GPB(4)  */
	reg = readl(GPBPUD) & (~0x300);
	writel(reg | 0x000, GPBPUD);

	/* Set GPB(4) (clock) output low */
	reg = readl(GPBDAT) & (~0x10);
	writel(reg | 0x00, GPBDAT);

	/* Set GPR(3) as input (Q output) */
	reg = readl(GPRCON0) & (~0xF000);
	writel(reg | 0x0000, GPRCON0);

	is_setup = 1;
}

void flipflop_set(int v)
{
	u32 reg;

	flipflop_setup();

	/* Read GPR(3) (Q output) */
	reg = readl(GPRDAT) & 0x8;

	if( (v) != (!!reg) ){	
		/* Pulse GPB(4) clock output, this will toggle the state of the Q output, 
		   which should be high */	
		reg = readl(GPBDAT) & (~0x10);	
		writel(reg | 0x10, GPBDAT);
		udelay(10);
		reg = readl(GPBDAT) & (~0x10);	
		writel(reg | 0x00, GPBDAT);
		udelay(10);		
	}
}

int flipflop_get(void)
{
	u32 reg;

	flipflop_setup();

	/* Read GPR(3) (Q output) */
	reg = readl(GPRDAT) & 0x8;
	/* if the flag was unset (in userland) then last boot passed! */
	return (reg == 0);
}

#endif /* CONFIG_SW_FLIPFLOP */

