/*
 * (C) Copyright 2002
 * Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Marius Groeger <mgroeger@sysgo.de>
 *
 * (C) Copyright 2002
 * David Mueller, ELSOFT AG, <d.mueller@elsoft.ch>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <bootscript.h>
#include <common.h>
#include <regs.h>
#include <asm/io.h>

DECLARE_GLOBAL_DATA_PTR;

/* ------------------------------------------------------------------------- */
#define DM9000_Tacs	(0x0)	// 0clk		address set-up
#define DM9000_Tcos	(0x4)	// 4clk		chip selection set-up
#define DM9000_Tacc	(0xE)	// 14clk	access cycle
#define DM9000_Tcoh	(0x1)	// 1clk		chip selection hold
#define DM9000_Tah	(0x4)	// 4clk		address holding time
#define DM9000_Tacp	(0x6)	// 6clk		page mode access cycle
#define DM9000_PMC	(0x0)	// normal(1data)page mode configuration

extern int system_bootmode;	/* Bootmode from common/main.c */

/* Here is the boot logic as HUSH script.
 */
extern char bootscript[];

static inline void delay(unsigned long loops)
{
	__asm__ volatile ("1:\n" "subs %0, %1, #1\n" "bne 1b":"=r" (loops):"0"(loops));
}

/*
 * Miscellaneous platform dependent initialisations
 */

/* Provide power to the dock so we get console on the iPod connector */
static void enable_console(void)
{
	u32 reg;

	/* Set GPR(1) as output (ACCESSORY_PWREN->3V3_RDS) */
	reg = readl(GPRCON0) & (~0xF0);
	writel(reg | 0x10, GPRCON0);

	/* Set disable pullup/down GPR(1)  */
	reg = readl(GPRPUD) & (~0xC);
	writel(reg, GPRPUD);

	/* Set GPR(1) output high */
	reg = readl(GPRDAT);
	writel(reg | 2, GPRDAT);
}

int board_init(void)
{
	char bootcmd[32];
	char ulong_str[16]; /* Needs to be as long as "0x12345678" + '\0' */

	enable_console();

	gd->bd->bi_arch_number = MACH_TYPE;
	gd->bd->bi_boot_params = (PHYS_SDRAM_1+0x100);

	setenv("bootdev", "1");

	if ( readl(RST_STAT) & BIT0 ) {
		/* HW_RESET */
		setenv("sysboot_mode","cold");
	} else {
		setenv("sysboot_mode","watchdog");
	}

	IMPORT_BOOTSCRIPT(preboot_plat, preboot_plat);
#ifdef CONFIG_DEBUG_BUILD
	IMPORT_BOOTSCRIPT(preboot_debug, preboot);
	IMPORT_BOOTSCRIPT(bootcmd_debug, bootcmd);
	IMPORT_BOOTSCRIPT(altbootcmd_debug, altbootcmd);
	IMPORT_BOOTSCRIPT(preboot, preboot.default);
	IMPORT_BOOTSCRIPT(bootcmd, bootcmd.default);
	IMPORT_BOOTSCRIPT(altbootcmd, altbootcmd.default);
#else
	IMPORT_BOOTSCRIPT(preboot, preboot);
	IMPORT_BOOTSCRIPT(bootcmd, bootcmd);
	IMPORT_BOOTSCRIPT(altbootcmd, altbootcmd);
#endif

	/* Set up the environment */
	sprintf(ulong_str, "%p", CFG_LOAD_ADDR);
	setenv("loadaddr", ulong_str);

	sprintf(ulong_str, "%p", CFG_FDT_ADDR);
	setenv("fdaddr", ulong_str);

	sprintf(ulong_str, "%#x", MEMADDR_KERN_LEN);
	setenv("kernel.maxsize", ulong_str);

	setenv("kernel.console", "ttySAC2");

	return 0;
}

int dram_init(void)
{
	DECLARE_GLOBAL_DATA_PTR;

	gd->bd->bi_dram[0].start = PHYS_SDRAM_1;
	gd->bd->bi_dram[0].size = PHYS_SDRAM_1_SIZE;

	return 0;
}

#ifdef BOARD_LATE_INIT
#if defined(CONFIG_BOOT_NAND)
int board_late_init (void)
{
	uint *magic = (uint*)(PHYS_SDRAM_1);
	char boot_cmd[100];

	if ((0x24564236 == magic[0]) && (0x20764316 == magic[1])) {
		sprintf(boot_cmd, "nand erase 0 40000;nand write %08x 0 40000", PHYS_SDRAM_1 + 0x8000);
		magic[0] = 0;
		magic[1] = 0;
		printf("\nready for self-burning U-Boot image\n\n");
		setenv("bootdelay", "0");
		setenv("bootcmd", boot_cmd);
	}

	return 0;
}
#elif defined(CONFIG_BOOT_MOVINAND)
int board_late_init (void)
{
	uint *magic = (uint*)(PHYS_SDRAM_1);
	char boot_cmd[100];
	int hc;

	hc = (magic[2] & 0x1) ? 1 : 0;

	if ((0x24564236 == magic[0]) && (0x20764316 == magic[1])) {
		sprintf(boot_cmd, "movi init %d %d;movi write u-boot %08x", magic[3], hc, PHYS_SDRAM_1 + 0x8000);
		magic[0] = 0;
		magic[1] = 0;
		printf("\nready for self-burning U-Boot image\n\n");
		setenv("bootdelay", "0");
		setenv("bootcmd", boot_cmd);
	}

	return 0;
}
#else
int board_late_init (void)
{
	return 0;
}
#endif
#endif

#ifdef CONFIG_DISPLAY_BOARDINFO
int checkboard(void)
{
	printf("Board:   Havana\n");
	return (0);
}
#endif

#ifdef CONFIG_ENABLE_MMU
ulong virt_to_phy_smdk6440(ulong addr)
{
	if ((0xc0000000 <= addr) && (addr < 0xc4000000))
		return (addr - 0xc0000000 + 0x20000000);
	else
		printf("do not support this address : %08lx\n", addr);

	return addr;
}
#endif

#if defined(CONFIG_CMD_NAND) && defined(CFG_NAND_LEGACY)
#include <linux/mtd/nand.h>
extern struct nand_chip nand_dev_desc[CFG_MAX_NAND_DEVICE];
void nand_init(void)
{
	nand_probe(CFG_NAND_BASE);
        if (nand_dev_desc[0].ChipID != NAND_ChipID_UNKNOWN) {
                print_size(nand_dev_desc[0].totlen, "\n");
        }
}
#endif
