/*
 * (C) Copyright 2002
 * Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Marius Groeger <mgroeger@sysgo.de>
 *
 * (C) Copyright 2002
 * David Mueller, ELSOFT AG, <d.mueller@elsoft.ch>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <bootscript.h>
#include <common.h>
#include <regs.h>
#include <asm/io.h>

DECLARE_GLOBAL_DATA_PTR;

extern int system_bootmode;	/* Bootmode from common/main.c */

static void __inline__ enable_sdcard(void)
{
	u32 reg;

	/* Set pin to output in GPIO CON */
	reg = readl(GPPCON) & ~0x30000;
	writel(reg | 0x1000, GPPCON);

	/* Switch off pullup/downs */
	reg = readl(GPPPUD) & ~0x30000;
	writel(reg | 0x0, GPPPUD);

	/* Set output to high */
	reg = readl(GPPDAT) & ~0x100;
	writel(reg | 0x1, GPPDAT);
}

int board_init(void)
{
	char bootcmd[32];
	char ulong_str[16]; /* Needs to be as long as "0x12345678" + '\0' */

	gd->bd->bi_arch_number = MACH_TYPE_TORINO_S;
	gd->bd->bi_boot_params = (PHYS_SDRAM_1+0x100);

	setenv("bootdev", "1");

	if ( readl(RST_STAT) & BIT0 ) {
		/* HW_RESET */
		setenv("sysboot_mode","cold");
	} else {
		setenv("sysboot_mode","watchdog");
	}

	enable_sdcard();

	IMPORT_BOOTSCRIPT(preboot_plat, preboot_plat);
#ifdef CONFIG_DEBUG_BUILD
	IMPORT_BOOTSCRIPT(preboot_debug, preboot);
	IMPORT_BOOTSCRIPT(bootcmd_debug, bootcmd);
	IMPORT_BOOTSCRIPT(altbootcmd_debug, altbootcmd);
	IMPORT_BOOTSCRIPT(preboot, preboot.default);
	IMPORT_BOOTSCRIPT(bootcmd, bootcmd.default);
	IMPORT_BOOTSCRIPT(altbootcmd, altbootcmd.default);
#else
	IMPORT_BOOTSCRIPT(preboot, preboot);
	IMPORT_BOOTSCRIPT(bootcmd, bootcmd);
	IMPORT_BOOTSCRIPT(altbootcmd, altbootcmd);
#endif

	/* Set up the environment */
	sprintf(ulong_str, "%p", CFG_LOAD_ADDR);
	setenv("loadaddr", ulong_str);

	sprintf(ulong_str, "%p", CFG_FDT_ADDR);
	setenv("fdaddr", ulong_str);

	sprintf(ulong_str, "%#x", MEMADDR_KERN_LEN);
	setenv("kernel.maxsize", ulong_str);

	setenv("kernel.console", "ttySAC3");

	return 0;
}

int dram_init(void)
{
	DECLARE_GLOBAL_DATA_PTR;

	gd->bd->bi_dram[0].start = PHYS_SDRAM_1;
	gd->bd->bi_dram[0].size = PHYS_SDRAM_1_SIZE;

	return 0;
}

#ifdef BOARD_LATE_INIT
int board_late_init (void)
{
	return 0;
}
#endif

#ifdef CONFIG_DISPLAY_BOARDINFO
int checkboard(void)
{
	printf("Board:      Seoul\n");
	return (0);
}
#endif

#ifdef CONFIG_ENABLE_MMU
ulong virt_to_phy_smdk6430(ulong addr)
{
	if ((0xc0000000 <= addr) && (addr < 0xc8000000))
		return (addr - 0xc0000000 + 0x50000000);
	else
		printf("do not support this address : %08lx\n", addr);

	return addr;
}
#endif

