/*
 * (C) Copyright 2008
 * TomTom International BV <www.tomtom.com>
 * Martin Jackson <martin.jackson@tomtom.com>
 * Based on:
 *
 * (C) Copyright 2004
 * esd gmbh <www.esd-electronics.com>
 * Reinhard Arlt <reinhard.arlt@esd-electronics.com>
 *
 * made from cmd_reiserfs by
 *
 * (C) Copyright 2003 - 2004
 * Sysgo Real-Time Solutions, AG <www.elinos.com>
 * Pavel Bartusek <pba@sysgo.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 */

/*
 * Ext2fs support
 */
#include <common.h>

#ifdef CONFIG_CMD_PART
#include <command.h>

/* #define	PART_DEBUG */

#ifdef	PART_DEBUG
#define	PRINTF(fmt,args...)	printf (fmt ,##args)
#else
#define PRINTF(fmt,args...)
#endif

int do_getpart (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	int dev, part;
	block_dev_desc_t *dev_desc=NULL;
	char str_part[4];

	if (argc < 5) {
		printf ("Usage:\n%s\n", cmdtp->usage);
		return 1;
	}

	dev = (int)simple_strtoul (argv[4], NULL, 16);
	dev_desc = get_dev (argv[3], dev);

	if (dev_desc == NULL) {
		printf ("\n** Block device %s %d not supported\n", argv[1], dev);
		return 1;
	}

	PRINTF ("Using device %s %#X\n", argv[3], dev);

	if (search_part (dev_desc, &part, argv[2])) {
		printf ("** Error search_part **\n");
		return 1;
	}

	PRINTF ("Found partition on %s %#X:%#X\n", argv[3], dev, part);

	sprintf (str_part, "%x", part);
	setenv (argv[1], str_part);

	return 0;
}

U_BOOT_CMD(
	getpart,	5,	1,	do_getpart,
	"getpart - Search for a partition\n",
	"<name> <attribute> <interface> <dev>\n"
	"    - Search for partition in 'dev' on 'interface' having 'attribute'"
	" and store in environment variable 'name'. Attribute is one of:\n"
	"\tfat  - find first FAT partition\n"
	"\text2 - find first EXT2 partition\n"
	"\tboot - find first bootable partition\n"
	"\tfd   - find first factory data partition\n"
);

#endif	/* CONFIG_CMD_PART */

