/*
 * (C) Copyright 2001
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * (C) Copyright 2009 TomTom BV <http://www.tomtom.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/* #define DEBUG */

/*
 * Command to set/activate the watchdog.
 */
#include <config.h>
#include <common.h>
#include <command.h>

#if defined(CONFIG_HW_WATCHDOG)
#if defined(CONFIG_CMD_WATCHDOG)

#include <watchdog.h>
#ifndef __NON_LEGACY
# include <flipflop.h>
#endif

static int do_watchdog (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	char *disable_wdog;

	if (argc != 2) {
		printf ("Usage:\n%s\n", cmdtp->usage);
		return 1;
	}

	if (getenv_yesno("disable_watchdog")) {
		debug("Watchdog disabled, ignoring\n");
		return 0;
	}

	if (strcmp(argv[1], "on") == 0) {
		hw_watchdog_activate(1);
#ifndef __NON_LEGACY
		flipflop_set(1);
#endif
	} else if (strcmp(argv[1], "off") == 0) {
		hw_watchdog_activate(0);
#ifndef __NON_LEGACY
		flipflop_set(0);
#endif
	}

	return 0;
}

U_BOOT_CMD(
 	watchdog,	2,	0,	do_watchdog,
 	"watchdog- [de]activate watchdog timer and fail boot flag\n",
	"watchdog [ on | off ]\n"
	"    - activate or deactivate watchdog timer and fail boot flag\n"
);

#endif	/* CONFIG_CMD_WATCHDOG */
#endif  /* CONFIG_HW_WATCHDOG */
