/*****************************************************************************
*  Copyright 2008 Broadcom Corporation.  All rights reserved.
*
*  Unless you and Broadcom execute a separate written software license
*  agreement governing use of this software, this software is licensed to you
*  under the terms of the GNU General Public License version 2, available at
*  http://www.gnu.org/copyleft/gpl.html (the "GPL").
*
*  Notwithstanding the above, under no circumstances may you combine this
*  software in any way with any other Broadcom software provided under a
*  license other than the GPL, without Broadcom's express prior written
*  consent.
*
*****************************************************************************/

#include <common.h>
#include <config.h>

#ifdef CFG_BCM4760_UART
#define UART_PHYS		CFG_BCM4760_UART_COM

#define __UARTREG(o)     (*((volatile u32 *)(UART_PHYS+(o))))

/* Register definitions */
#define UARTDR		0x000	/* RW Data Register */
#define		DR_DATA		0x00FF	/* Receive Data Character */
#define		DR_FE		0x0100	/* Framing Error */
#define		DR_PE		0x0200	/* Parity Error */
#define		DR_BE		0x0400	/* Break Error */
#define		DR_OE		0x0800	/* Overrun Error */
#define UARTRSR		0x004	/* R  Receive Status Register */
#define		RSR_FE		0x0001	/* Framing Error */
#define		RSR_PE		0x0002	/* Parity Error */
#define		RSR_BE		0x0004	/* Break Error */
#define		RSR_OE		0x0008	/* Overrun Error */
#define UARTECR		0x004	/*  W Error Clear Register */
#define UARTFR		0x018	/* R  Flag Register */
#define		FR_CTS		0x0001	/* Clear To Send */
#define		FR_DSR		0x0002	/* Data Set Ready */
#define		FR_DCD		0x0004	/* Data Carrier Detect */
#define		FR_BUSY		0x0008	/* UART Busy */
#define		FR_RXFE		0x0010	/* Receive FIFO Empty */
#define		FR_TXFF		0x0020	/* Transmit FIFO Full */
#define		FR_RXFF		0x0040	/* Receive FIFO Full */
#define		FR_TXFE		0x0080	/* Transmit FIFO Empty */
#define		FR_RI		0x0100	/* Ring Indicator */
#define UARTILPR	0x020	/* RW IrDA Low Power Counter Register */
#define UARTIBRD	0x024	/* RW Integer Baud Rate Register */
#define UARTFBRD	0x028	/* RW Fractional Baud Rate Register */
#define UARTLCR		0x02C	/* RW Line Control Register */
#define		LCR_BRK		0x0001	/* Send Break */
#define		LCR_PEN		0x0002	/* Parity Enable */
#define		LCR_EPS		0x0004	/* Even Parity Select */
#define		LCR_STP2	0x0008	/* Two Stop Bits Select */
#define		LCR_FEN		0x0010	/* Enable FIFOs */
#define		LCR_WLEN	0x0060	/* Word Length */
#define			LCR_WLEN_5BIT	0x0000
#define			LCR_WLEN_6BIT	0x0020
#define			LCR_WLEN_7BIT	0x0040
#define			LCR_WLEN_8BIT	0x0060
#define		LCR_SPS		0x0080	/* Stick Parity Select */
#define UARTCR		0x030	/* RW Control Register */
#define		CR_UARTEN	0x0001	/* UART Enable */
#define		CR_SIREN	0x0002	/* SIR Enable */
#define		CR_SIRLP	0x0004	/* SIR Low Power IrDA mode */
#define		CR_LBE		0x0080	/* LoopBack Enable */
#define		CR_TXE		0x0100	/* Transmit Enable */
#define		CR_RXE		0x0200	/* Receive Enable */
#define		CR_DTR		0x0400	/* Data Transmit Ready */
#define		CR_RTS		0x0800	/* Request to Send */
#define		CR_OUT1		0x1000
#define		CR_OUT2		0x2000
#define		CR_RTSEn	0x4000	/* RTS Hardware Flow Control Enable */
#define UARTIFLS	0x034	/* RW Interrupt FIFO Level Select Register */
#define		IFLS_RXIFLSEL	0x0038
#define			IFLS_RXIFSEL18	0x0000
#define			IFLS_RXIFSEL14	0x0008
#define			IFLS_RXIFSEL12	0x0010
#define			IFLS_RXIFSEL34	0x0018
#define			IFLS_RXIFSEL78	0x0020
#define		IFLS_TXIFLSEL	0x0007
#define			IFLS_TXIFLSEL18	0x0000
#define			IFLS_TXIFLSEL14	0x0001
#define			IFLS_TXIFLSEL12	0x0002
#define			IFLS_TXIFLSEL34	0x0003
#define			IFLS_TXIFLSEL78	0x0004
#define UARTIMSC	0x038	/* RW Interrupt Mask Set/Clr Register */
#define		IMSC_RIMIM	0x0001	/* nUARTRI Modem Interrupt Mask */
#define		IMSC_CTSMIM	0x0002	/* nUARTCTS Modem Interrupt Mask */
#define		IMSC_DCDMIM	0x0004	/* nUARTDCD Modem Interrupt Mask */
#define		IMSC_DSRMIM	0x0008	/* nUARTDSR Modem Interrupt Mask */
#define		IMSC_RXIM	0x0010	/* Receive Interrupt Mask */
#define		IMSC_TXIM	0x0020	/* Transmit Interrupt Mask */
#define		IMSC_RTIM	0x0040	/* Receive Timeout Interrupt Mask */
#define		IMSC_FEIM	0x0080	/* Framing Error Interrupt Mask */
#define		IMSC_PEIM	0x0100	/* Parity Error Interrupt Mask */
#define		IMSC_BEIM	0x0200	/* Break Error Interrupt Mask */
#define		IMSC_OEIM	0x0400	/* Overrun Error Interrupt Mask */
#define UARTRIS		0x03C	/* R  UART Raw Interrupt Status Register */
#define		RIS_RIRMIS	0x0001	/* nUARTRI Modem Interrupt Status */
#define		RIS_CTSRMIS	0x0002	/* nUARTCTS Modem Interrupt Status */
#define		RIS_DCDRMIS	0x0004	/* nUARTDCD Modem Interrupt Status */
#define		RIS_DSRRMIS	0x0008	/* nUARTDSR Modem Interrupt Status */
#define		RIS_RXRIS	0x0010	/* Receive Raw Interrupt State */
#define		RIS_TXRIS	0x0020	/* Transmit Raw Interrupt Status */
#define		RIS_RTRIS	0x0040	/* Receive Timeout Interrupt Status */
#define		RIS_FERIS	0x0080	/* Framing Error Interrupt Status */
#define		RIS_PERIS	0x0100	/* Parity Error Interrupt Status */
#define		RIS_BERIS	0x0200	/* Break Error Interrupt Status */
#define		RIS_OERIS	0x0400	/* Overrun Error Interrupt Status */
#define	UARTMIS		0x040	/* R  Masked Interrupt Status Register */
#define		MIS_RIMMIS	0x0001	/* nUARTRI Modem Masked Interrupt Status */
#define		MIS_CTSMMIS	0x0002	/* nUARTCTS Modem Masked Interrupt Status */
#define		MIS_DCDMMIS	0x0004	/* nUARTDCD Modem Masked Interrupt Status */
#define		MIS_DSRMMIS	0x0008	/* nUARTDSR Modem Masked Interrupt Status */
#define		MIS_RXMIS	0x0010	/* Receive Raw Masked Interrupt State */
#define		MIS_TXMIS	0x0020	/* Transmit Raw Masked Interrupt Status */
#define		MIS_RTMIS	0x0040	/* Receive Timeout Masked Interrupt Status */
#define		MIS_FEMIS	0x0080	/* Framing Error Masked Interrupt Status */
#define		MIS_PEMIS	0x0100	/* Parity Error Masked Interrupt Status */
#define		MIS_BEMIS	0x0200	/* Break Error Masked Interrupt Status */
#define		MIS_OEMIS	0x0400	/* Overrun Error Masked Interrupt Status */
#define UARTICR		0x044	/*  W Interrupt Clear Register */
#define		ICR_RIMIC	0x0001	/* nUARTRI Modem Interrupt Clear */
#define		ICR_CTSMIC	0x0002	/* nUARTCTS Modem Interrupt Clear */
#define		ICR_DCDMIC	0x0004	/* nUARTDCD Modem Masked Interrupt Clear */
#define		ICR_DSRMIC	0x0008	/* nUARTDSR Modem Masked Interrupt Clear */
#define		ICR_RXMIC	0x0010	/* Receive Raw Masked Interrupt State */
#define		ICR_TXMIC	0x0020	/* Transmit Raw Masked Interrupt Status */
#define		ICR_RTMIC	0x0040	/* Receive Timeout Masked Interrupt Status */
#define		ICR_FEMIC	0x0080	/* Framing Error Masked Interrupt Status */
#define		ICR_PEMIC	0x0100	/* Parity Error Masked Interrupt Status */
#define		ICR_BEMIC	0x0200	/* Break Error Masked Interrupt Status */
#define		ICR_OEMIC	0x0400	/* Overrun Error Masked Interrupt Status */
#define UARTDMACR	0x048	/* RW DMA Control Register */
#define		DMACR_RXDMAE	0x0001	/* Receive DMA Enable */
#define		DMACR_TXDMAE	0x0002	/* Transmit DMA Enable */
#define		DMACR_DMAONERR	0x0004	/* DMA On Error */


DECLARE_GLOBAL_DATA_PTR;

void serial_setbrg(void)
{
    u32 temp;
	u32 divider;
	u32 remainder;
	u32 fraction;

	if (!gd->baudrate)
		gd->baudrate = CONFIG_BAUDRATE;

	/*
	 * Set Baud Rate
	 *
	 * IBRD = UART_CLK / (16 * Baud)
	 * FBRD = ROUND((64 * MOD(UART_CLK, (16 * Baud))) / (16 * Baud))
	 */
	temp = 16 * CONFIG_BAUDRATE;
	divider = CFG_BCM4760_UART_CLK / temp;
	remainder = CFG_BCM4760_UART_CLK % temp;
	fraction = (4 * remainder) / CONFIG_BAUDRATE;

	__UARTREG(UARTIBRD) = divider;
	__UARTREG(UARTFBRD) = fraction;

	/*
	 * NOTE: Must be written last, after IBRD and FBRD
	 *
	 * Set the UART 8 data bits, 1 stop bit, no parity, and FIFO enable.
	 */
	__UARTREG(UARTLCR) = LCR_WLEN_8BIT | LCR_FEN;
}

int serial_getc(void)
{
	/*
	 * wait if FIFO is empty
	 */
	while (__UARTREG(UARTFR) & FR_RXFE)
	{
		/* wait */
	}

	/*
	 * send back the next character
	 */
	return (__UARTREG(UARTDR) & DR_DATA);
}

void serial_putc(const char c)
{
	/*
	 * wait if FIFO is full
	 */
	while (__UARTREG(UARTFR) & FR_TXFF)
	{
		/* wait */
	}

	__UARTREG(UARTDR) = c;

	/*
	 * If \n, also do \r
	 */
	if (c == '\n')
	{
		serial_putc ('\r');
	}
}

/*
 * Test whether a character is in the RX buffer
 */
int serial_tstc(void)
{
	/*
	 * If receive fifo is empty or errors (overrun / break /parity / framing), return false
	 */
	if (__UARTREG(UARTFR) & FR_RXFE) { 
		return 0;
	} else {
		return 1;
	}
}

void serial_puts(const char *s)
{
	while (*s)
	{
		serial_putc (*s++);
	}
}

/*
 * Initialize the serial port with the given baudrate. The settings
 * are always 8 data bits, no parity, 1 stop bit, no start bits.
 */
int serial_init(void)
{
	/*
	 * Mask off all interrupts
	 */
	__UARTREG(UARTIMSC) = 0;

	/*
	 * Enable the UART, but no Transmit or Receive
	 */
	__UARTREG(UARTCR) = CR_UARTEN;

	/*
	 * Clear pending interrupts
	 */
	__UARTREG(UARTICR) = __UARTREG(UARTRIS);

	/*
	 * Clear Framing, Parity, Break, and Overrun Errors
	 */
	__UARTREG(UARTECR) = 0;

	/*
	 * Clear DMA Control
	 */
	__UARTREG(UARTDMACR) = 0;

	/*
	 * Initialize IrDA Low Power Counter
	 */
	__UARTREG(UARTILPR) = 0;

	/*
	 * Initialize Interrupt FIFO Level
	 */
	__UARTREG(UARTIFLS) = 0;

	/*
	 * Set Baud Rate
	 */
	serial_setbrg();

	/*
	 * Enable UART, with both Transmit and Receive
	 */
	__UARTREG(UARTCR) = CR_UARTEN | CR_TXE | CR_RXE;

	return 0;
}
#endif /* CFG_BCM4760_UART */

