/*
 * pubkey2c.c - export a rsa public key for inclusion in BL1
 *
 * 2010-04-15 Ard Biesheuvel <ard.biesheuvel@tomtom.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <arpa/inet.h>

#include <openssl/pem.h>

static void print_num( FILE *fd, BIGNUM *num)
{
	int i, n = BN_num_bytes(num);
	unsigned char *buf = malloc(n);

	BN_bn2bin( num, buf );

	fprintf(fd, "{");

	for (i = 0; i < n; i += 1) {
		if ( !(i % 16))
			fprintf(fd, "\n\t");
		fprintf(fd, "0x%02x, ", buf[n-i-1]);
	}

	while (i++ % 4)
		fprintf(fd, "0x00, ");

	fprintf(fd, "\n},");

	free(buf);
}

int main(int argc, char **argv)
{
	RSA *rsa;
	FILE *fp;
	int i;

	if (argc != 2)
		return 1;

	if ( !(fp = fopen( argv[1], "r")))
		return 1;

	if ( !(rsa = PEM_read_RSA_PUBKEY(fp, NULL, NULL, NULL))) {
		fprintf(stderr, "Could not read public key from file\n");
		return 1;
	}

	printf("RawRSAPublicKey const __early_data BL2_Key = {\n");

	print_num( stdout, rsa->n );
	print_num( stdout, rsa->e );

	printf("};\n");

	fclose( fp );

	return 0;
}

