/*
 *  Copyright (C) 2007-08 SiRF Tecnology, Inc
 *  based on 
 *  linux/drivers/mmc/sdhci.h - Secure Digital Host Controller Interface driver
 *  Copyright (C) 2005-2007 Pierre Ossman, All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 */

/*
 * Controller registers
 */
#ifndef _TITAN_MMC_H_
#define _TITAN_MMC_H_

#define TITANMMC_PHY_BASE			(0x56000000)

#define TITANMMC_DMA_ADDRESS	0x00

#define TITANMMC_BLOCK_SIZE	0x04
#define  TITANMMC_MAKE_BLKSZ(dma, blksz) (((dma & 0x7) << 12) | (blksz & 0xFFF))

#define TITANMMC_BLOCK_COUNT	0x06

#define TITANMMC_ARGUMENT		0x08

#define TITANMMC_TRANSFER_MODE	0x0C
#define  TITANMMC_TRNS_DMA		0x01
#define  TITANMMC_TRNS_BLK_CNT_EN	0x02
#define  TITANMMC_TRNS_ACMD12	0x04
#define  TITANMMC_TRNS_READ	0x10
#define  TITANMMC_TRNS_MULTI	0x20

#define TITANMMC_COMMAND		0x0E
#define  TITANMMC_CMD_RESP_MASK	0x03
#define  TITANMMC_CMD_CRC		0x08
#define  TITANMMC_CMD_INDEX	0x10
#define  TITANMMC_CMD_DATA		0x20

#define  TITANMMC_CMD_RESP_NONE	0x00
#define  TITANMMC_CMD_RESP_LONG	0x01
#define  TITANMMC_CMD_RESP_SHORT	0x02
#define  TITANMMC_CMD_RESP_SHORT_BUSY 0x03

#define TITANMMC_MAKE_CMD(c, f) (((c & 0xff) << 8) | (f & 0xff))

#define TITANMMC_RESPONSE		0x10

#define TITANMMC_BUFFER		0x20

#define TITANMMC_PRESENT_STATE	0x24
#define  TITANMMC_CMD_INHIBIT	0x00000001
#define  TITANMMC_DATA_INHIBIT	0x00000002
#define  TITANMMC_DOING_WRITE	0x00000100
#define  TITANMMC_DOING_READ	0x00000200
#define  TITANMMC_SPACE_AVAILABLE	0x00000400
#define  TITANMMC_DATA_AVAILABLE	0x00000800
#define  TITANMMC_CARD_PRESENT	0x00010000
#define  TITANMMC_WRITE_PROTECT	0x00080000

#define TITANMMC_HOST_CONTROL 	0x28
#define  TITANMMC_CTRL_LED		0x01
#define  TITANMMC_CTRL_4BITBUS	0x02
#define  TITANMMC_CTRL_HISPD	0x04
#define  TITANMMC_CTRL_8BITBUS	0x08


#define TITANMMC_POWER_CONTROL	0x29
#define  TITANMMC_POWER_ON		0x01
#define  TITANMMC_POWER_180	0x0A
#define  TITANMMC_POWER_300	0x0C
#define  TITANMMC_POWER_330	0x0E

#define TITANMMC_BLOCK_GAP_CONTROL	0x2A

#define TITANMMC_WALK_UP_CONTROL	0x2B

#define TITANMMC_CLOCK_CONTROL	0x2C
#define  TITANMMC_DIVIDER_SHIFT	8
#define  TITANMMC_CLOCK_CARD_EN	0x0004
#define  TITANMMC_CLOCK_INT_STABLE	0x0002
#define  TITANMMC_CLOCK_INT_EN	0x0001

#define TITANMMC_TIMEOUT_CONTROL	0x2E

#define TITANMMC_SOFTWARE_RESET	0x2F
#define  TITANMMC_RESET_ALL	0x01
#define  TITANMMC_RESET_CMD	0x02
#define  TITANMMC_RESET_DATA	0x04

#define TITANMMC_INT_STATUS	0x30
#define TITANMMC_INT_ENABLE	0x34
#define TITANMMC_SIGNAL_ENABLE	0x38
#define  TITANMMC_INT_RESPONSE	0x00000001
#define  TITANMMC_INT_DATA_END	0x00000002
#define  TITANMMC_INT_DMA_END	0x00000008
#define  TITANMMC_INT_SPACE_AVAIL	0x00000010
#define  TITANMMC_INT_DATA_AVAIL	0x00000020
#define  TITANMMC_INT_CARD_INSERT	0x00000040
#define  TITANMMC_INT_CARD_REMOVE	0x00000080
#define  TITANMMC_INT_CARD_INT	0x00000100
#define  TITANMMC_INT_TIMEOUT	0x00010000
#define  TITANMMC_INT_CRC		0x00020000
#define  TITANMMC_INT_END_BIT	0x00040000
#define  TITANMMC_INT_INDEX	0x00080000
#define  TITANMMC_INT_DATA_TIMEOUT	0x00100000
#define  TITANMMC_INT_DATA_CRC	0x00200000
#define  TITANMMC_INT_DATA_END_BIT	0x00400000
#define  TITANMMC_INT_BUS_POWER	0x00800000
#define  TITANMMC_INT_ACMD12ERR	0x01000000

#define  TITANMMC_INT_NORMAL_MASK	0x00007FFF
#define  TITANMMC_INT_ERROR_MASK	0xFFFF8000

#define  TITANMMC_INT_CMD_MASK	(TITANMMC_INT_RESPONSE | TITANMMC_INT_TIMEOUT | \
		TITANMMC_INT_CRC | TITANMMC_INT_END_BIT | TITANMMC_INT_INDEX)
#define  TITANMMC_INT_DATA_MASK	(TITANMMC_INT_DATA_END | TITANMMC_INT_DMA_END | \
		TITANMMC_INT_DATA_AVAIL | TITANMMC_INT_SPACE_AVAIL | \
		TITANMMC_INT_DATA_TIMEOUT | TITANMMC_INT_DATA_CRC | \
		TITANMMC_INT_DATA_END_BIT)

#define TITANMMC_ACMD12_ERR	0x3C

/* 3E-3F reserved */

#define TITANMMC_CAPABILITIES	0x40
#define  TITANMMC_TIMEOUT_CLK_MASK	0x0000003F
#define  TITANMMC_TIMEOUT_CLK_SHIFT 0
#define  TITANMMC_TIMEOUT_CLK_UNIT	0x00000080
#define  TITANMMC_CLOCK_BASE_MASK	0x00003F00
#define  TITANMMC_CLOCK_BASE_SHIFT	8
#define  TITANMMC_MAX_BLOCK_MASK	0x00030000
#define  TITANMMC_MAX_BLOCK_SHIFT  16
#define  TITANMMC_CAN_DO_HISPD	0x00200000
#define  TITANMMC_CAN_DO_DMA	0x00400000
#define  TITANMMC_CAN_VDD_330	0x01000000
#define  TITANMMC_CAN_VDD_300	0x02000000
#define  TITANMMC_CAN_VDD_180	0x04000000

/* 44-47 reserved for more caps */

#define TITANMMC_MAX_CURRENT	0x48

/* 4C-4F reserved for more max current */

/* 50-FB reserved */

#define TITANMMC_SLOT_INT_STATUS	0xFC

#define TITANMMC_HOST_VERSION	0xFE
#define  TITANMMC_VENDOR_VER_MASK	0xFF00
#define  TITANMMC_VENDOR_VER_SHIFT	8
#define  TITANMMC_SPEC_VER_MASK	0x00FF
#define  TITANMMC_SPEC_VER_SHIFT	0


#define UNSTUFF_BITS(resp,start,size)					\
	({								\
		const int __size = size;				\
		const u32 __mask = (__size < 32 ? 1 << __size : 0) - 1;	\
		const int __off = 3 - ((start) / 32);			\
		const int __shft = (start) & 31;			\
		u32 __res;						\
									\
		__res = resp[__off] >> __shft;				\
		if (__size + __shft > 32)				\
			__res |= resp[__off-1] << ((32 - __shft) % 32);	\
		__res & __mask;						\
	})

#endif
