/*****************************************************************************
*  Copyright 2008 Broadcom Corporation.  All rights reserved.
*
*  Unless you and Broadcom execute a separate written software license
*  agreement governing use of this software, this software is licensed to you
*  under the terms of the GNU General Public License version 2, available at
*  http://www.gnu.org/copyleft/gpl.html (the "GPL").
*
*  Notwithstanding the above, under no circumstances may you combine this
*  software in any way with any other Broadcom software provided under a
*  license other than the GPL, without Broadcom's express prior written
*  consent.
*
*****************************************************************************/

#ifndef __ASM_ARCH_BCM_SDHCI_H
#define __ASM_ARCH_BCM_SDHCI_H

/*
 * SDHCI Registers/Fields
 */
#define SDHCI_SDMA_SYS_ADDR_LOFF				(0)

#define SDHCI_BLK_SIZE_WOFF						( 0x4)
#define SDHCI_BLK_CNT_WOFF						( 0x6)
#define SDHCI_ARGUMENT_LOFF						( 0x8)

#define SDHCI_CMD_XFER_MODE_LOFF				( 0xC)
#define SDHCI_XFER_MODE_WOFF					( 0xC)

#define SDHCI_COMMAND_WOFF						( 0xE)
#define  		SDHCI_CMD_RESP_NONE				( 0x00 )
#define  		SDHCI_CMD_RESP_LONG				( 0x01 )
#define  		SDHCI_CMD_RESP_SHORT			( 0x02 )
#define  		SDHCI_CMD_RESP_SHORT_BUSY		( 0x03 )

#define SDHCI_CMD_CRC_CHK_ENABLE	0x8
#define SDHCI_CMD_INDEX_CHK_ENABLE	0x10
#define SDHCI_DATA_PRESENT	0x20

#define SDHCI_DMA_ENABLE 	1
#define SDHCI_BLK_CNT_ENABLE (1 << 1)
#define SDHCI_ACMD12_ENABLE	(1 << 2)
#define SDHCI_DATA_XFER_READ (1 << 4)
#define SDHCI_DATA_XFER_WRITE (0)
#define SDHCI_MULTI_BLK		(1 << 5)

#define SDHCI_CMD_COMPLETE	1
#define SDHCI_XFER_COMPLETE	2
#define SDHCI_DMA_INT		 (1 << 3 )

#define SDHCI_SW_RESET_ALL 0x1
#define SDHCI_SW_RESET_CMDLINE 0x2
#define SDHCI_SW_RESET_DATLINE 0x4

#define SDHCI_SDBUS_POWER			(0x1)
#define SDHCI_SDBUS_VOLTAGE_3_3V	(0x7 << 1)

#define SDHCI_INTERNAL_CLOCK_ENABLE	0x1
#define SDHCI_INTERNAL_CLOCK_STABLE 	(0x1 << 1)
#define SDHCI_SD_CLOCK_ENABLE	(0x1 << 2)

#define SDHCI_CARD_INSERTED	(1 << 18)
#define SDHCI_CMD_TIMEOUT_ERR (1 << 16)
#define SDHCI_HOST_SDMA_BUFFER_SIZE_512K (7 << 12)

#define SDHCI_RESPONSE0_WOFF					( 0x10)
#define SDHCI_RESPONSE1_WOFF					( 0x12)
#define SDHCI_RESPONSE10_LOFF					( 0x10)
#define SDHCI_RESPONSE2_WOFF					( 0x14)
#define SDHCI_RESPONSE3_WOFF					( 0x16)
#define SDHCI_RESPONSE32_LOFF					( 0x14)
#define SDHCI_RESPONSE4_WOFF					( 0x18)
#define SDHCI_RESPONSE5_WOFF					( 0x1a)
#define SDHCI_RESPONSE54_LOFF					( 0x18)
#define SDHCI_RESPONSE6_WOFF					( 0x1c)
#define SDHCI_RESPONSE7_WOFF					( 0x1e)
#define SDHCI_RESPONSE76_LOFF					( 0x1c)

#define SDHCI_BUF_DATA_PORT_LOFF				( 0x20)

#define SDHCI_PRESENT_STATE_LOFF				( 0x24)


#define SDHCI_HOST_CTRL_BOFF					( 0x28)

#define SDHCI_POWER_CTRL_BOFF					( 0x29)


#define SDHCI_BLOCK_GAP_CTRL_BOFF				( 0x2a)


#define SDHCI_WAKEUP_CTRL_BOFF					( 0x2b)


#define SDHCI_CLOCK_CTRL_WOFF					( 0x2c)
#define SDHCI_TIMEOUT_CLOCK_CTRL_LOFF			( 0x2c)
#define			SDHCI_SD_CLOCK_DIV_1			( 0x0000 )
#define			SDHCI_SD_CLOCK_DIV_2			( 0x0100 )
#define			SDHCI_SD_CLOCK_DIV_4			( 0x0200 )
#define			SDHCI_SD_CLOCK_DIV_8			( 0x0400 )
#define			SDHCI_SD_CLOCK_DIV_16			( 0x0800 )
#define			SDHCI_SD_CLOCK_DIV_32			( 0x1000 )
#define			SDHCI_SD_CLOCK_DIV_64			( 0x2000 )
#define			SDHCI_SD_CLOCK_DIV_128			( 0x4000 )
#define			SDHCI_SD_CLOCK_DIV_256			( 0x8000 )

#define SDHCI_TIMEOUT_CTRL_BOFF					( 0x2e )


#define SDHCI_SW_RESET_BOFF						( 0x2f)

#define SDHCI_INT_STS_LOFF						( 0x30)
#define SDHCI_INT_STS_EN_LOFF					( 0x34)
#define SDHCI_INT_SIG_EN_LOFF					( 0x38)
#define SDHCI_NORM_INT_STS_WOFF					( 0x30)
#define SDHCI_NORM_INT_STS_EN_WOFF				( 0x34)
#define SDHCI_NORM_INT_SIG_EN_WOFF				( 0x38)


#define SDHCI_ERR_INT_STS_WOFF					( 0x32)
#define SDHCI_ERR_INT_STS_EN_WOFF				( 0x36)
#define SDHCI_ERR_INT_SIG_EN_WOFF				( 0x3a)


#define SDHCI_ACMD12_ERR_STS_WOFF				( 0x3c)


#define SDHCI_CAPABILITIES_LOFF					( 0x40)


#define SDHCI_MAX_CURRENT_CAPABILITIES_LOFF		( 0x48)


#define SDHCI_FORCE_EVENT_ACMD12_ERR_STS_WOFF	( 0x50)

#define SDHCI_FORCE_EVENT_ERR_INT_STS_WOFF		( 0x52)

#define SDHCI_F_OD					0x8001
#define SDHCI_F_PP					0x8005
#define SDHCI_F_OD_HS				0x0101
#define SDHCI_F_PP_HS				0x0105
#define SDHCI_F_OD_VHS				0x0001
#define SDHCI_F_PP_VHS				0x0005
#define SDHCI_F_OD_LS				0x0401
#define SDHCI_F_PP_LS				0x0405
#define SDHCI_INTERNAL_CLK_STABLE	(1 << 1) /* bit 1 */

#endif /* __ASM_ARCH_BCM_SDHCI_H */
