/*****************************************************************************
* Copyright 2006 - 2008 Broadcom Corporation.  All rights reserved.
*
* Unless you and Broadcom execute a separate written software license
* agreement governing use of this software, this software is licensed to you
* under the terms of the GNU General Public License version 2, available at
* http://www.broadcom.com/licenses/GPLv2.php (the "GPL"). 
*
* Notwithstanding the above, under no circumstances may you combine this
* software in any way with any other Broadcom software provided under a
* license other than the GPL, without Broadcom's express prior written
* consent.
*****************************************************************************/



#ifndef __BOARD_CONFIG_H
#define __BOARD_CONFIG_H

/*
 * High Level Configuration Options
 */
#define CONFIG_ARM1136		1	/* This is an arm1136 CPU core */

#include <asm/arch/bcm2153.h>		/* get chip and board defs */

/* input clock of PLL */
#define CONFIG_SYS_CLK_FREQ	32768

#undef CONFIG_USE_IRQ			/* no support for IRQs */
#define CONFIG_MISC_INIT_R

#define CONFIG_CMDLINE_TAG	1	/* enable passing of ATAGs */
#define CONFIG_SETUP_MEMORY_TAGS	1
#define CONFIG_INITRD_TAG	1

/*-----------------------------------------------------------------------
 * Physical Memory Map
 */
#define CONFIG_NR_DRAM_BANKS	1
#define PHYS_SDRAM_1		CFG_GLOBAL_RAM_BASE
#define PHYS_SDRAM_1_SIZE	HW_SDRAM_SIZE

/*-----------------------------------------------------------------------
 * Stack sizes
 *
 * The stack sizes are set up in start.S using the settings below
 */
#define CONFIG_STACKSIZE	SZ_128K	/* regular stack */
#ifdef CONFIG_USE_IRQ
#define CONFIG_STACKSIZE_IRQ	SZ_4K	/* IRQ stack */
#define CONFIG_STACKSIZE_FIQ	SZ_4K	/* FIQ stack */
#endif

/*
 * Size of malloc() pool
 */
#define CFG_ENV_SIZE		16384	/* Total Size of Environment Sector */
#define CFG_MALLOC_LEN		(CFG_ENV_SIZE + SZ_128K + SZ_16K)
#define CFG_GBL_DATA_SIZE	128	/* size in bytes reserved for initial data */

/*
 * Hardware drivers
 */

/*
 * NS16550 Configuration
 */
#define CFG_NS16550
#define CFG_NS16550_SERIAL
#define CFG_NS16550_REG_SIZE	(-4)
#define CFG_NS16550_CLK		UARTCLK
#define CFG_NS16550_COM1	HW_UART_A_BASE

/*
 * select serial console configuration
 */
#define CONFIG_SERIAL1		1
#define CONFIG_SERIAL2		2

/* allow to overwrite serial and ethaddr */
#define CONFIG_ENV_OVERWRITE
#define CONFIG_CONS_INDEX	1
#define CONFIG_BAUDRATE		115200
#define CFG_BAUDRATE_TABLE	{9600, 19200, 38400, 57600, 115200, 230400, 460800, 921600}

/*
 * Miscellaneous configurable options
 */
#define CFG_LONGHELP		/* undef to save memory */
#define CFG_PROMPT		"# "
#define CFG_CBSIZE		2048	/* Console I/O Buffer Size */
/* Print Buffer Size */
#define CFG_PBSIZE		(CFG_CBSIZE+sizeof(CFG_PROMPT)+16)
#define CFG_MAXARGS		16	/* max number of command args */
#define CFG_BARGSIZE		CFG_CBSIZE	/* Boot Argument Buffer Size */

#define CFG_MEMTEST_START	PHYS_SDRAM_1	/* memtest works on */
#define CFG_MEMTEST_END		(CFG_MEMTEST_START + PHYS_SDRAM_1_SIZE)

#undef	CFG_CLKS_IN_HZ		/* everything, incl board info, in Hz */

/* default load address */
#define CFG_LOAD_ADDR		(PHYS_SDRAM_1 + 0x00800000 - sizeof(image_header_t))
#define CFG_ATAG_ADDR		(PHYS_SDRAM_1 + 0x100)

#define CFG_TIMERBASE		REG_GPT_BASE1
#define CFG_HZ			32768 /* GPT1 */

/* Boot delay disabled*/
#define CONFIG_BOOTDELAY	5

/* This will save the env to flash if env is empty */

#define CONFIG_BOOTCOMMAND	"run setup_env; saveenv; run bootcmd"
#define CONFIG_ZERO_BOOTDELAY_CHECK	1
#define CONFIG_BOOTFILE		"uImage"


/*
 *  CONFIG_BOOTARGS_SETUP is executed on first run.
 *  It will override CONFIG_BOOTARGS settings. This is done because mtdparts
 *  is generated by bad block information and needs to be scanned per board.
 */
#ifdef NFS_BOOT_DEFAULTS
#define CONFIG_BOOTARGS		"mem=32M console=ttyS0,115200n8 noinitrd root=/dev/nfs rw nfsroot=128.247.77.158:/home/a0384864/wtbu/rootfs ip=dhcp"
#else
#if (CONFIG_GLOBAL_ROOT_FILE_SYSTEM == INITRAMFS)
#define CONFIG_BOOTARGS		"console=ttyS0,115200n8 mem=96M"
#define CONFIG_BOOTARGS_SETUP "setup_bootargs=setenv bootargs console=ttyS0,115200n8 mem=96M\0"
#else
#define CONFIG_BOOTARGS		"console=ttyS0,115200n8 mem=96M ${mtdparts} root=/dev/mtdblock6 rw rootfstype=jffs2"
#define CONFIG_BOOTARGS_SETUP "setup_bootargs=setenv bootargs console=ttyS0,115200n8 mem=96M ${mtdparts} root=/dev/mtdblock6 rw rootfstype=jffs2\0"
#endif
#endif

//#define CONFIG_NETMASK		255.255.255.0
//#define CONFIG_IPADDR			127.0.0.1
//#define CONFIG_SERVERIP		127.0.0.1

/*
 * FLASH and environment organization
 */
#define CFG_NO_FLASH
/* #define CFG_ENV_IS_NOWHERE */
#define CFG_ENV_IS_IN_NAND	1
//#define CFG_ENV_NAND_NOTALIGNED
#define CFG_ENV_OFFSET		0x80000	/* environment starts here  */

/*
 *  Board NAND Info.
 */

#define CFG_NAND_BASE		HW_NAND_BASE
#define CFG_MAX_NAND_DEVICE	1

#define NAND_MAX_CHIPS		1
/*
 * BCM Nand Driver and settings
 */

#define CONFIG_MTD_NAND_BCM_UMI
#define CONFIG_MTD_NAND_BCM_UMI_HWECC	1

/*
 * JFFS2 Settings
 */
#define CONFIG_JFFS2_CMDLINE
#define CONFIG_JFFS2_NAND
#define CONFIG_JFFS2_PART_OFF 0x2600000
#define CONFIG_JFFS2_PART_SIZE 0x1400000
#define CONFIG_JFFS2_DEV "nand0"

/*
 * SD/MMC Settings
 */
#define CFG_SDHCI_MAXDEVICE	2

/*
 * MTD Partitions
 */

#define CFG_NAND_MTD_KERNEL_NAME "bcm_umi-nand"
#define MTDIDS_DEFAULT "nand0=bcm_umi-nand"
#define MTDPARTS_DEFAULT "mtdparts=bcm_umi-nand:0x4000@0x0(boot1),0x80000@0x4000(boot2),0x1F7C000@0x84000(CP),0x200000@0x2000000(kernel)ro,0x80000@0x2200000(nvdata),0x380000@0x2280000(data),0x2800000@0x2600000(root1),0x1200000@0x4e00000(root2),-(root3)"

/*
 * Don't show In/Out device
 */
#define CFG_CONSOLE_INFO_QUIET	1


/*
 * BOOTP options
 */

/*
#define CONFIG_BOOTP_BOOTFILESIZE
#define CONFIG_BOOTP_BOOTPATH
#define CONFIG_BOOTP_GATEWAY
#define CONFIG_BOOTP_HOSTNAME
#define CONFIG_BOOTP_SUBNETMASK
*/


/*
 * Command line configuration.
 */
#include <config_cmd_default.h>
#define CONFIG_CMD_NAND
#define CONFIG_CMD_JFFS2
#define CONFIG_CMD_SDHCI
#define CONFIG_CMD_FAT
 #undef CONFIG_CMD_NET
#undef CONFIG_CMD_NFS
#undef CONFIG_CMD_FLASH
#undef CONFIG_CMD_IMLS
/*
#define CONFIG_CMD_ASKENV
#define CONFIG_CMD_DATE
#define CONFIG_CMD_DHCP
#define CONFIG_CMD_DTT
#define CONFIG_CMD_DIAG
#define CONFIG_CMD_EEPROM
#define CONFIG_CMD_ELF
#define CONFIG_CMD_I2C
#define CONFIG_CMD_IRQ
#define CONFIG_CMD_MII
#define CONFIG_CMD_NAND
#define CONFIG_CMD_NET
#define CONFIG_CMD_NFS
#define CONFIG_CMD_PCI
#define CONFIG_CMD_PING
#define CONFIG_CMD_REGINFO
#define CONFIG_CMD_SDRAM
#ifdef CONFIG_460EX
#define CONFIG_CMD_EXT2
#define CONFIG_CMD_FAT
#define CONFIG_CMD_USB
#endif
*/

/* Partitions */
//#define CONFIG_MAC_PARTITION
#define CONFIG_DOS_PARTITION
//#define CONFIG_ISO_PARTITION

/*
 * BCM Custom
 */
/* Enables fwupg command -- soft reset with PUMR=1
 * for UART fwupg through brom
 * */
#define CONFIG_CMD_BCM_DLMODE
/*
 * These are used to init the env vars on first boot.
 */
#define CONFIG_CMD_FIXPARTS
#ifdef CONFIG_CMD_FIXPARTS
#define CONFIG_SETUP_ENV "setup_env=fixparts; run setup_bootargs setup_clean\0"
#else
#define CONFIG_SETUP_ENV "setup_env=mtdparts default; run setup_bootargs setup_clean\0"
#endif

#define CONFIG_EXTRA_ENV_SETTINGS		\
	CONFIG_SETUP_ENV \
	CONFIG_BOOTARGS_SETUP \
	"setup_clean=setenv setup_env; setenv setup_clean\0" \
	"num_partitions=9\0" \
	"part0_size=0x4000\0" \
	"part1_size=0x80000\0" \
	"part2_size=0x1F7C000\0" \
	"part3_size=0x200000\0" \
	"part4_size=0x80000\0" \
	"part5_size=0x380000\0" \
	"part6_size=0x2800000\0" \
	"part7_size=0x1200000\0" \
	"part8_size=-\0" \
	"part0_name=Boot1\0" \
	"part1_name=U-Boot\0" \
	"part2_name=CP\0" \
	"part3_name=kernel\0" \
	"part4_name=nvdata\0" \
	"part5_name=data\0" \
	"part6_name=root1\0" \
	"part7_name=root2\0" \
	"part8_name=root3\0" \
	"part3_flag=ro\0"


#if (CFG_GLOBAL_U_BOOT_OVERRIDE == 1)
#include "uboot_config.h"
#endif

#endif /* __BOARD_CONFIG_H */
