/*
 * U-Boot configuration for Havana device
 *
 * Copyright (C) 2010 TomTom International B.V.
 * 
 ************************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 * 02110-1301, USA.
 ************************************************************************
 */

#ifndef __BOARD_CONFIG_H
#define __BOARD_CONFIG_H

/* Temp */
#define CFG_STACK_SIZE		512*1024

/************************************************************************
* Machine settings                                                      *
************************************************************************/

#define CONFIG_S5P6440		1		/* in a SAMSUNG S5P6440 SoC */
#define CONFIG_S5P64XX		1		/* in a SAMSUNG S5P64XX Family  */
#define CONFIG_HAVANA		1		/* on a TomTom HAVANA Board */

/************************************************************************
* Platform specific shell commands                                      *
************************************************************************/

#define CONFIG_CMD_MOVINAND

/************************************************************************
* Hardware settings                                                     *
************************************************************************/

#define CONFIG_S3C_MMC_BOOT			/* Boot from MMC */

#define MEMORY_BASE_ADDRESS	(0x20000000)
#define CONFIG_SYS_CLK_FREQ	12000000	/* the Havana has 12MHz input clock */

#define CONFIG_ENABLE_MMU
#ifdef CONFIG_ENABLE_MMU
#define virt_to_phys(x)	virt_to_phy_smdk6440(x)
#else
#define virt_to_phys(x)	(x)
#endif

#define CONFIG_MEMORY_UPPER_CODE
#define CONFIG_INCLUDE_TEST
#define BOARD_LATE_INIT
#define CONFIG_HW_WATCHDOG
#define CONFIG_CMD_FLIPFLOP

/*
 * Architecture magic and machine type
 */
#define MACH_TYPE		2468
#define UBOOT_MAGIC		(0x43090000 | MACH_TYPE)

/* Power Management is enabled */
#define CONFIG_PM

#define CONFIG_DISPLAY_CPUINFO
#define CONFIG_DISPLAY_BOARDINFO

#undef CONFIG_SKIP_RELOCATE_UBOOT
#undef CONFIG_USE_NOR_BOOT
#define CONFIG_SERIAL3          1	/* we use UART2 (aka SERIAL3) on havana */
#define CONFIG_RTC_S5P64XX	1

/* the PWM TImer 4 uses a counter of 15625 for 10 ms, so we need */
/* it to wrap 100 times (total 1562500) to get 1 sec. */
#define CFG_HZ			1562500		/* at PCLK 50MHz */

//#define CONFIG_CLK_667_166_83
//#define CONFIG_CLK_533_133_66
#define CONFIG_CLK_500_166_83
//#define CONFIG_CLK_400_133_66
//#define CONFIG_CLK_400_100_50



#define set_pll(mdiv, pdiv, sdiv)	(1<<31 | mdiv<<16 | pdiv<<8 | sdiv)

#if defined(CONFIG_CLK_667_166_83)
#define APLL_MDIV		667
#define APLL_PDIV		6
#define APLL_SDIV		1
#elif defined(CONFIG_CLK_533_133_66)
#define APLL_MDIV		533
#define APLL_PDIV		6
#define APLL_SDIV		1
#elif defined(CONFIG_CLK_500_166_83)
#define APLL_MDIV		500
#define APLL_PDIV		3
#define APLL_SDIV		2
#elif defined(CONFIG_CLK_400_100_50)
#define APLL_MDIV		400
#define APLL_PDIV		3
#define APLL_SDIV		2
#elif defined(CONFIG_CLK_400_133_66)
#define APLL_MDIV		400
#define APLL_PDIV		3
#define APLL_SDIV		2
#else
#error "Not Support Fequency or Mode!! you have to setup right configuration."
#endif

#if 1
/* Target frequency : 390MHz */
#define MPLL_MDIV		260
#define MPLL_PDIV		2
#define MPLL_SDIV		2
#else
/* Target frequency : 400MHz */
/* Beware, this breaks USB... (50MHz instead of 48MHz) */
#define MPLL_MDIV		400
#define MPLL_PDIV		3
#define MPLL_SDIV		2
#endif

#define APLL_VAL	set_pll(APLL_MDIV, APLL_PDIV, APLL_SDIV)
/* prevent overflow */
#define Startup_APLL	(CONFIG_SYS_CLK_FREQ/(APLL_PDIV<<APLL_SDIV)*APLL_MDIV)

#define MPLL_VAL	set_pll(MPLL_MDIV, MPLL_PDIV, MPLL_SDIV)
/* prevent overflow */
#define Startup_MPLL	((CONFIG_SYS_CLK_FREQ)/(MPLL_PDIV<<MPLL_SDIV)*MPLL_MDIV)

#if defined(CONFIG_CLK_667_166_83)
#define	Startup_PCLKdiv		1
#define Startup_HCLKdiv		3
#define Startup_MPLLdiv		0
#define Startup_ARMdiv		0
#elif defined(CONFIG_CLK_533_133_66)
#define	Startup_PCLKdiv		1
#define Startup_HCLKdiv		3
#define Startup_MPLLdiv		0
#define Startup_ARMdiv		0
#elif defined(CONFIG_CLK_500_166_83)
#define	Startup_PCLKdiv		1
#define Startup_HCLKdiv		2
#define Startup_MPLLdiv		0
#define Startup_ARMdiv		0
#elif defined(CONFIG_CLK_400_100_50)
#define	Startup_PCLKdiv		1
#define Startup_HCLKdiv		3
#define Startup_MPLLdiv		0
#define Startup_ARMdiv		0
#elif defined(CONFIG_CLK_400_133_66)
#define	Startup_PCLKdiv		1
#define Startup_HCLKdiv		2
#define Startup_MPLLdiv		0
#define Startup_ARMdiv		0
#else
#error "Not Support Fequency or Mode!! you have to setup right configuration."
#endif

#define CLK_DIV_VAL	((Startup_PCLKdiv<<12)|(Startup_HCLKdiv<<8)|(Startup_MPLLdiv<<4)|Startup_ARMdiv)

#define Startup_HCLK	((Startup_APLL)/(Startup_HCLKdiv+1))

/*-----------------------------------------------------------------------
 * Physical Memory Map
 */


/* Memory Parameters Havana board*/
/* DDR Parameters, mclk=133Mhz*/
#define DDR_tREFRESH		7800		/* ns */
#define DDR_tRAS		42		/* ns (min: 42ns)*/
#define DDR_tRC 		60		/* ns (min: 60ns)*/
#define DDR_tRCD		18		/* ns (min: 18ns)*/
#define DDR_tRFC		80		/* ns (min: 80ns)*/
#define DDR_tRP 		18		/* ns (min: 18ns)*/
#define DDR_tRRD		12		/* ns (min: 12ns)*/
#define DDR_tWR 		12		/* ns (min: 12ns)*/
#define DDR_tXSR		120		/* ns (min: 120ns)*/
#define DDR_CASL		3		/* CAS Latency 3 */
#define DDR_tWTR		19

/*
 * mDDR memory configuration
 */
#define DMC_DDR_BA_EMRS 	2
#define DMC_DDR_MEM_CASLAT	3
#define DMC_DDR_CAS_LATENCY	(DDR_CASL<<1)						//6   Set Cas Latency to 3
#define DMC_DDR_t_DQSS		1							// Min 0.75 ~ 1.25

#define DMC_DDR_t_RAS		(((Startup_HCLK / 1000 * DDR_tRAS) - 1) / 1000000 + 1)	//7, Min 45ns
#define DMC_DDR_t_RC		(((Startup_HCLK / 1000 * DDR_tRC) - 1) / 1000000 + 1) 	//10, Min 67.5ns
#define DMC_DDR_t_RCD		(((Startup_HCLK / 1000 * DDR_tRCD) - 1) / 1000000 + 1) 	//4,5(TRM), Min 22.5ns
#define DMC_DDR_schedule_RCD	((DMC_DDR_t_RCD - 3) << 3)
#define DMC_DDR_t_RFC		(((Startup_HCLK / 1000 * DDR_tRFC) - 1) / 1000000 + 1) 	//11,18(TRM) Min 80ns
#define DMC_DDR_schedule_RFC	((DMC_DDR_t_RFC - 3) << 5)
#define DMC_DDR_t_RP		(((Startup_HCLK / 1000 * DDR_tRP) - 1) / 1000000 + 1) 	//4, 5(TRM) Min 22.5ns
#define DMC_DDR_schedule_RP	((DMC_DDR_t_RP - 3) << 3)
#define DMC_DDR_t_RRD		(((Startup_HCLK / 1000 * DDR_tRRD) - 1) / 1000000 + 1)	//3, Min 15ns
#define DMC_DDR_t_WR		(((Startup_HCLK / 1000 * DDR_tWR) - 1) / 1000000 + 1)	//Min 15ns
#define DMC_DDR_t_WTR		(((Startup_HCLK / 1000 * DDR_tWTR) - 1) / 1000000 + 1)
#define DMC_DDR_t_XSR		(((Startup_HCLK / 1000 * DDR_tXSR) - 1) / 1000000 + 1)	//17, Min 120ns
#define DMC_DDR_t_ESR		DMC_DDR_t_XSR
#define DMC_DDR_REFRESH_PRD	(((Startup_HCLK / 1000 * DDR_tREFRESH) - 1) / 1000000) 	// TRM 2656
#define DMC_DDR_USER_CONFIG	1							// 2b01 : mDDR
#define DMC_DDR_t_RTP		1
/*DDR_PHYCONTROL0*/
#define DMC_DDR_CTRL_FORCE	106
#define DMC_DDR_CTRL_INC	16
#define DMC_DDR_CTRL_STRT_POINT	16
#define DMC_DDR_CTRL_DQS_DELAY	0
#define DMC_DDR_CTRL_DFDQS	0
#define DMC_DDR_CTRL_HALF	0
#define DMC_DDR_CTRL_DLL_ON	1
#define DMC_DDR_CTRL_STRT	1
/*DDR_PHYCONTROL1*/
#define DMC_DDR_DRV_TYPE	1 // pull down all
#define DMC_DDR_CTRL_OFFSETC	4
#define DMC_DDR_CTRL_REF	0xF
#define DMC_DDR_CTRL_SHIFTC	4 // T/8 45' shift
/*DDR_CONCONTROL*/
#define DMC_DDR_TIMEOUT_CNT	4095
#define DMC_DDR_RD_FETCH	1
#define DMC_DDR_DQ_SWAP		0
#define DMC_DDR_CHIP1_EMPTY	0
#define DMC_DDR_CHIP0_EMPTY	0
#define DMC_DDR_DRV_EN		0
#define DMC_DDR_CTC_RTR_GAP_EN	0
#define DMC_DDR_AREF_DIS	0 /*auto refresh disabled*/
#define DMC_DDR_AREF_EN		1 /*auto refresh enabled*/
#define DMC_DDR_OUT_OF		1
/*DDR_MEMCONTROL*/
#define DMC_DDR_BL		2 /*BL =4*/
#define DMC_DDR_CHIP		0
#define DMC_DDR_MEM_WIDTH	2
#define DMC_DDR_MEM_TYPE	1
#define DMC_DDR_ADD_LAT_PALL	0
#define DMC_DDR_DSREF_EN	0
#define DMC_DDR_TP_EN		0
#define DMC_DDR_DPWRDN_TYPE	0
#define DMC_DDR_DPWRDN_EN	0
#define DMC_DDR_CLK_STOP_EN	0
/*DDR_PRECHCONFIG*/
#define DMC_DDR_TP_CNT		20
#define DMC_DDR_CHIP1_POLICY	0
#define DMC_DDR_CHIP0_POLICY	0
/*DDR_PWRDNCONFIG*/
#define DMC_DDR_DSREF_CYC	16
#define DMC_DDR_DPWRDN_CYC	4
/*DDR_TIMINGPOWER*/
#define DMC_DDR_t_MRD		2							//Min 2 tck
#define DMC_DDR_t_CKE		2
#define DMC_DDR_t_XP		4							//1tck + tIS(1.5ns)
#define DMC_DDR_t_FAW		14


#define DDR_PHYCONTROL0		( DMC_DDR_CTRL_STRT | (DMC_DDR_CTRL_DLL_ON << 1) | (DMC_DDR_CTRL_HALF << 2) | \
				(DMC_DDR_CTRL_DFDQS << 3) | (DMC_DDR_CTRL_DQS_DELAY << 4) | \
				(DMC_DDR_CTRL_STRT_POINT << 8) | (DMC_DDR_CTRL_INC << 16) | \
				(DMC_DDR_CTRL_FORCE << 24) )
#define DDR_PHYCONTROL1		( DMC_DDR_CTRL_SHIFTC | (DMC_DDR_CTRL_REF << 4) | (DMC_DDR_CTRL_OFFSETC << 8) | \
				(DMC_DDR_DRV_TYPE << 15) )
#define DDR_PHYCONTROL2		0 // should be zero
/*Auto refresh disabled*/
#define DDR_CONCONTROL_DIS	( (DMC_DDR_TIMEOUT_CNT << 16) | (DMC_DDR_RD_FETCH << 12) | (DMC_DDR_DQ_SWAP << 10) | \
				(DMC_DDR_CHIP1_EMPTY << 9) | (DMC_DDR_CHIP0_EMPTY << 8) | (DMC_DDR_DRV_EN << 7) | \
				(DMC_DDR_CTC_RTR_GAP_EN << 6) | (DMC_DDR_AREF_DIS << 5) | (DMC_DDR_OUT_OF << 4) )
/*Auto refresh enabled*/
#define DDR_CONCONTROL_EN	( DDR_CONCONTROL_DIS | (DMC_DDR_AREF_EN << 5) )
#define DDR_MEMCONTROL		( DMC_DDR_CLK_STOP_EN | (DMC_DDR_DPWRDN_EN << 1) | (DMC_DDR_DPWRDN_TYPE << 2) | \
				(DMC_DDR_TP_EN << 4) | (DMC_DDR_DSREF_EN << 5) | (DMC_DDR_ADD_LAT_PALL << 6) | \
				(DMC_DDR_MEM_TYPE << 8) | (DMC_DDR_MEM_WIDTH << 12) | (DMC_DDR_CHIP << 16) | \
				(DMC_DDR_BL << 20) )
#define DDR_PRECHCONTROL	( DMC_DDR_CHIP0_POLICY | (DMC_DDR_CHIP1_POLICY << 8) | (DMC_DDR_TP_CNT << 24) )	
#define DDR_TIMINGPOWER		( DMC_DDR_t_MRD | (DMC_DDR_t_CKE << 4) | (DMC_DDR_t_XP << 8) | (DMC_DDR_t_XSR << 16) |\
				(DMC_DDR_t_FAW << 24) ) 
#define DDR_TIMINGROW		( DMC_DDR_t_RAS | (DMC_DDR_t_RC << 6) | (DMC_DDR_t_RCD << 12) | \
				(DMC_DDR_t_RP << 16) | (DMC_DDR_t_RRD << 20) | (DMC_DDR_t_RFC << 24) )
#define DDR_TIMINGDATA		( DMC_DDR_MEM_CASLAT << 16 | (DMC_DDR_t_RTP <<20) | (DMC_DDR_t_WR << 24) |\
				(DMC_DDR_t_WTR << 28) | 0x0204)
#define DMC_DDR_PWRDNCONFIG	( DMC_DDR_DPWRDN_CYC | (DMC_DDR_DSREF_CYC << 16) )

/* list of direct commands*/
#define DDR_DIRECTCMD_NOP	0x07000000 /*exit from active/ precharge power down or deep power down*/
#define DDR_DIRECTCMD_REFA	0x05000000 /*auto refresh*/
#define DDR_DIRECTCMD_PALL	0x01000000 /*all banks precharge*/
#define DDR_DIRECTCMD_MRS_EMRS	0x00000000 /*mode register settings*/

#define DDR_DIRECTCMD_VALA	0x32 /* chip0 MRS, CL=3, sequential Burst, BL=4 */
//#define DDR_DIRECTCMD_VALB	0x20000 /* chip0 EMRS, PASR=Full Array, DS=FULL */
#define DDR_DIRECTCMD_VALB	0x20020 /* chip0 EMRS, PASR=Full Array, DS=1/2 */
//#define DDR_DIRECTCMD_VALB	0x20040 /* chip0 EMRS, PASR=Full Array, DS=1/4 */
//#define DDR_DIRECTCMD_VALB	0x20060 /* chip0 EMRS, PASR=Full Array, DS=1/8 */

#define DDR_SET_MRS		( DDR_DIRECTCMD_MRS_EMRS | (DDR_DIRECTCMD_VALA) )
#define DDR_SET_EMRS		( DDR_DIRECTCMD_MRS_EMRS | (DDR_DIRECTCMD_VALB) )

#define CONFIG_NR_DRAM_BANKS	1	   /* we have 2 bank of DRAM */
#define PHYS_SDRAM_1		MEMORY_BASE_ADDRESS /* SDRAM Bank #1 */
#define PHYS_SDRAM_1_SIZE	SZ_64M
#define CONFIG_S5PMEMCFG0_VALUE	0x20F00212
#define TIMING_AREF		0x0000050F /*7.8us*133MHz=1038(0x40E), 16MHz=125(0x7D), 12MHz=93(0x5D)*/

/* total memory required by uboot */
#define CFG_UBOOT_SIZE		(2*1024*1024)

/* base address for uboot */
#ifdef CONFIG_ENABLE_MMU
#define CFG_UBOOT_BASE		(0xc3e00000)
#else
#define CFG_UBOOT_BASE		(0x23e00000)
#endif
#define CFG_PHY_UBOOT_BASE	(MEMORY_BASE_ADDRESS + 0x3e00000)
#define CONFIG_MMC

/************************************************************************
* Generic settings                                                      *
************************************************************************/

#include "plat-tomtom.conf"
#undef CONFIG_TT_HEADER

#endif /* __BOARD_CONFIG_H */

