/*
 * (C) Copyright 2000
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef	_SDHCI_H
#define _SDHCI_H

#define SDHCI_TIMEOUT 100 /* 1 ms */

#define CFG_SDHCI_BLOCK_SIZE		4096

/* Standard MMC commands (3.1)           type  argument     response */
   /* class 1 */
#define MMC_GO_IDLE_STATE         0   /* bc                          */
#define MMC_SEND_OP_COND          1   /* bcr  [31:0] OCR         R3  */
#define MMC_ALL_SEND_CID          2   /* bcr                     R2  */
#define MMC_SET_RELATIVE_ADDR     3   /* ac   [31:16] RCA        R1  */
#define MMC_SET_DSR               4   /* bc   [31:16] RCA            */
#define MMC_SWITCH                6   /* ac   [31:26] 0 [25:24] access
                  [23:16] index [15:8] value [7:3] 0 [2:0] cmd set   */
#define MMC_SELECT_CARD           7   /* ac   [31:16] RCA        R1  */
#define MMC_SEND_EXT_CSD          8   /* adtc                    R1  */
#define MMC_SEND_CSD              9   /* ac   [31:16] RCA        R2  */
#define MMC_SEND_CID             10   /* ac   [31:16] RCA        R2  */
#define MMC_READ_DAT_UNTIL_STOP  11   /* adtc [31:0] dadr        R1  */
#define MMC_STOP_TRANSMISSION    12   /* ac                      R1b */
#define MMC_SEND_STATUS          13   /* ac   [31:16] RCA        R1  */
#define MMC_GO_INACTIVE_STATE    15   /* ac   [31:16] RCA            */

  /* class 2 */
#define MMC_SET_BLOCKLEN         16   /* ac   [31:0] block len   R1  */
#define MMC_READ_SINGLE_BLOCK    17   /* adtc [31:0] data addr   R1  */
#define MMC_READ_MULTIPLE_BLOCK  18   /* adtc [31:0] data addr   R1  */

  /* class 3 */
#define MMC_WRITE_DAT_UNTIL_STOP 20   /* adtc [31:0] data addr   R1  */

  /* class 4 */
#define MMC_SET_BLOCK_COUNT      23   /* adtc [31:0] data addr   R1  */
#define MMC_WRITE_BLOCK          24   /* adtc [31:0] data addr   R1  */
#define MMC_WRITE_MULTIPLE_BLOCK 25   /* adtc                    R1  */
#define MMC_PROGRAM_CID          26   /* adtc                    R1  */
#define MMC_PROGRAM_CSD          27   /* adtc                    R1  */

  /* class 6 */
#define MMC_SET_WRITE_PROT       28   /* ac   [31:0] data addr   R1b */
#define MMC_CLR_WRITE_PROT       29   /* ac   [31:0] data addr   R1b */
#define MMC_SEND_WRITE_PROT      30   /* adtc [31:0] wpdata addr R1  */

  /* class 5 */
#define MMC_ERASE_GROUP_START    35   /* ac   [31:0] data addr   R1  */
#define MMC_ERASE_GROUP_END      36   /* ac   [31:0] data addr   R1  */
#define MMC_ERASE                38   /* ac                      R1b */

  /* class 9 */
#define MMC_FAST_IO              39   /* ac   <Complex>          R4  */
#define MMC_GO_IRQ_STATE         40   /* bcr                     R5  */

  /* class 7 */
#define MMC_LOCK_UNLOCK          42   /* adtc                    R1b */

  /* class 8 */
#define MMC_APP_CMD              55   /* ac   [31:16] RCA        R1  */
#define MMC_GEN_CMD              56   /* adtc [0] RD/WR          R1  */

/* SD commands                           type  argument     response */
  /* class 8 */
/* This is basically the same command as for MMC with some quirks. */
#define SD_SEND_RELATIVE_ADDR     3   /* bcr                     R6  */

/* CEATA commands                        type  argument      response */
  /* class 9 */
#define MMC_RW_MULTIPLE_REGISTER  60   /* adtc  complex       R1b/R1  */

#define MMC_RW_MULTIPLE_BLOCK     61   /* adtc  complex       R1b/R1  */


  /* Application commands */
#define SD_APP_SET_BUS_WIDTH      6   /* ac   [1:0] bus width    R1  */
#define SD_APP_OP_COND           41   /* bcr  [31:0] OCR         R3  */
#define SD_APP_SEND_SD_STATUS    13   /* adtc                    R1  */
#define SD_APP_SEND_SCR          51   /* adtc                    R1  */

#define SD_BUS_WIDTH_1				0
#define SD_BUS_WIDTH_4				2
#define SD_SWITCH_MODE_SET		0x80000000
#define SD_SWITCH_MODE_HIGHSPEED	0x00FFFFF1

/*
  MMC status in R1
  Type
    e : error bit
    s : status bit
    r : detected and set for the actual command response
    x : detected and set during command execution. the host must poll
            the card by sending status command in order to read these bits.
  Clear condition
    a : according to the card state
    b : always related to the previous command. Reception of
            a valid command will clear it (with a delay of one command)
    c : clear by read
 */

#define SD_SEND_IF_COND           8   /* bcr  [11:0] See below   R7  */
/*
 * SD_SEND_IF_COND argument format:
 *
 *	[31:12] Reserved (0)
 *	[11:8] Host Voltage Supply Flags
 *	[7:0] Check Pattern (0xAA)
 */


#define R1_OUT_OF_RANGE     (1 << 31)   /* er, c */
#define R1_ADDRESS_ERROR    (1 << 30)   /* erx, c */
#define R1_BLOCK_LEN_ERROR  (1 << 29)   /* er, c */
#define R1_ERASE_SEQ_ERROR      (1 << 28)   /* er, c */
#define R1_ERASE_PARAM      (1 << 27)   /* ex, c */
#define R1_WP_VIOLATION     (1 << 26)   /* erx, c */
#define R1_CARD_IS_LOCKED   (1 << 25)   /* sx, a */
#define R1_LOCK_UNLOCK_FAILED   (1 << 24)   /* erx, c */
#define R1_COM_CRC_ERROR    (1 << 23)   /* er, b */
#define R1_ILLEGAL_COMMAND  (1 << 22)   /* er, b */
#define R1_CARD_ECC_FAILED  (1 << 21)   /* ex, c */
#define R1_CC_ERROR     (1 << 20)   /* erx, c */
#define R1_ERROR        (1 << 19)   /* erx, c */
#define R1_UNDERRUN     (1 << 18)   /* ex, c */
#define R1_OVERRUN      (1 << 17)   /* ex, c */
#define R1_CID_CSD_OVERWRITE    (1 << 16)   /* erx, c, CID/CSD overwrite */
#define R1_WP_ERASE_SKIP    (1 << 15)   /* sx, c */
#define R1_CARD_ECC_DISABLED    (1 << 14)   /* sx, a */
#define R1_ERASE_RESET      (1 << 13)   /* sr, c */
#define R1_STATUS(x)            (x & 0xFFFFE000)
#define R1_CURRENT_STATE(x)     ((x & 0x00001E00) >> 9) /* sx, b (4 bits) */
#define R1_READY_FOR_DATA   (1 << 8)    /* sx, a */
#define R1_APP_CMD      (1 << 5)    /* sr, c */


struct mmc_command {
    u32         opcode;
    u32         arg;
    u32         resp[4];
    unsigned int        flags;      /* expected response type */
#define MMC_RSP_PRESENT (1 << 0)
#define MMC_RSP_136 (1 << 1)        /* 136 bit response */
#define MMC_RSP_CRC (1 << 2)        /* expect valid crc */
#define MMC_RSP_BUSY    (1 << 3)        /* card may send busy */
#define MMC_RSP_OPCODE  (1 << 4)        /* response contains opcode */
#define MMC_CMD_MASK    (3 << 5)        /* command type */
#define MMC_CMD_AC  (0 << 5)
#define MMC_CMD_ADTC    (1 << 5)
#define MMC_CMD_BC  (2 << 5)
#define MMC_CMD_BCR (3 << 5)

/*
 * These are the response types, and correspond to valid bit
 * patterns of the above flags.  One additional valid pattern
 * is all zeros, which means we don't expect a response.
 */
#define MMC_RSP_NONE    (0)
#define MMC_RSP_R1  (MMC_RSP_PRESENT|MMC_RSP_CRC|MMC_RSP_OPCODE)
#define MMC_RSP_R1B (MMC_RSP_PRESENT|MMC_RSP_CRC|MMC_RSP_OPCODE|MMC_RSP_BUSY)
#define MMC_RSP_R2  (MMC_RSP_PRESENT|MMC_RSP_136|MMC_RSP_CRC)
#define MMC_RSP_R3  (MMC_RSP_PRESENT)
#define MMC_RSP_R4  (MMC_RSP_PRESENT)
#define MMC_RSP_R6  (MMC_RSP_PRESENT|MMC_RSP_CRC)
#define MMC_RSP_R7	(MMC_RSP_PRESENT|MMC_RSP_CRC)

#define mmc_resp_type(cmd)  ((cmd)->flags & (MMC_RSP_PRESENT|MMC_RSP_136|MMC_RSP_CRC|MMC_RSP_BUSY|MMC_RSP_OPCODE))

/*
 * These are the command types.
 */
#define mmc_cmd_type(cmd)   ((cmd)->flags & MMC_CMD_MASK)

    unsigned int        retries;    /* max number of retries */
    unsigned int        error;      /* command error */

#define MMC_ERR_NONE    0
#define MMC_ERR_TIMEOUT 1
#define MMC_ERR_BADCRC  2
#define MMC_ERR_FIFO    3
#define MMC_ERR_FAILED  4
#define MMC_ERR_INVALID 5

	unsigned int       dd;
	unsigned int       dp;
	unsigned int       ac12;
	unsigned int	   dma_ena;

#if 0
    unsigned int        dbg_status;

    struct mmc_data     *data;      /* data segment associated with cmd */
    struct mmc_request  *mrq;       /* associated request */
#endif
};

#define MMC_SWITCH_ACCESS_COMMAND_SET 	0x00000000
#define MMC_SWITCH_ACCESS_SET_BITS 		0x01000000
#define MMC_SWITCH_ACCESS_CLEAR_BITS 	0x02000000
#define MMC_SWITCH_ACCESS_WRITE_BYTE 	0x03000000

#define MMC_RSP_CS_SWITCH_ERROR			0x00000080

#define MMC_SWITCH_INDEX_SHIFT			16
#define MMC_SWITCH_VALUE_SHIFT			8

#define EXT_CSD_SEC_COUNT           212

#define EXT_CSD_HIGH_SPEED_MODE     185            // @KP: add

#define EXT_CSD_BOOT_BUS_WIDTH      179            // @KP: add

#define EXT_CSD_BUS_WIDTH           183            // @KP: add
#define EXT_CSD_BUS_WIDTH_1         0x0
#define EXT_CSD_BUS_WIDTH_4         0x1
#define EXT_CSD_BUS_WIDTH_8         0x2

#define EXT_CSD_STRUCTURE        194
#define EXT_CSD_CARD_TYPE        196
#define EXT_CSD_CARD_TYPE_26MHZ   0x1
#define EXT_CSD_CARD_TYPE_52MHZ   0x3

struct mmc_cid {
    unsigned int        manfid;
    char            	prod_name[8];
    unsigned int        serial;
    unsigned short      oemid;
    unsigned short      year;
    unsigned char       hwrev;
    unsigned char       fwrev;
    unsigned char       month;
};

struct mmc_csd
{
    unsigned char       mmca_vsn;
    unsigned short      cmdclass;
    unsigned short      tacc_clks;
    unsigned int        tacc_ns;
    unsigned int        r2w_factor;
    unsigned int        max_dtr;
    unsigned int        read_blkbits;
    unsigned int        write_blkbits;
    unsigned int        capacity;
	unsigned int        read_partial:1;
	unsigned int		read_misalign:1;
	unsigned int		write_partial:1;
	unsigned int		write_misalign:1;
};

struct sd_scr {
    unsigned char       sda_vsn;
    unsigned char       bus_widths;
#define SD_SCR_BUS_WIDTH_1  (1<<0)
#define SD_SCR_BUS_WIDTH_4  (1<<2)
};

struct ceata_cfg {
    unsigned int        sector_bits;
    unsigned int        capacity_in_kb;
};

struct mmc_ext_csd {
	uint8_t csd_boot_bus_width;
	uint8_t csd_bus_width;
	uint8_t csd_card_type;
	uint8_t csd_structure_ver;
};

struct sd_scr2 {
	uint8_t		sda_vsn;
	uint8_t		bus_widths;
	uint16_t	reserved;
	uint32_t manufacture_data;
};

typedef struct _tag_sdhci_info_s {
	uint8_t hc_card;
	uint8_t sd_hcs;
	uint8_t card_type;
	uint8_t sdhci_ready;
	uint16_t card_rca;
	uint8_t csd_mmc_spec_vers;
	uint8_t csd_trans_speed;
	uint32_t blksz; 			/* block size */
	uint32_t max_clk;
	uint8_t *sdhci_buf;

	uint32_t sdhci_base;

    uint8_t csd_reg[16];
	struct sd_scr2 sd_scr_info;
	struct mmc_ext_csd mmc_ext_csd_info;
	uint8_t* sys_buf;
	uint8_t sys_buffer[CFG_SDHCI_BLOCK_SIZE+16];
} sdhci_info_t;


/*
 * Function Prototypes
 */
block_dev_desc_t* sdhci_get_dev(int dev);
void sdhci_init(int dev);
ulong sdhci_read(int dev, lbaint_t blknr, ulong blkcnt, void* buff);
ulong sdhci_write(int dev, lbaint_t blknr, ulong blkcnt, const void* buff);

#endif /* _SDHCI_H */
